/*
*  RAL -- Rubrica Addressbook Library
*  file: abook.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <libintl.h>
#include <glib.h>
#include <glib-object.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "abook.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "io.h"
#include "load.h"
#include "save.h"
#include "utils.h"
#include "error.h"

#define RUBRICA_FILE_FORMAT 4
// #define VERSION 4

/*
enum {
  RUBRICA_ABOOK_HAVE_NAME = 1,
  RUBRICA_ABOOK_NAME = 2,
  RUBRICA_ABOOK_PATH = 3
};
*/


struct _RubricaAbookPrivate {
  gboolean have_name;   
  gchar* name;          /* the addressbook (the file) name */
  gchar* path;          /* the addressbook (the file) path */

  GList* cards;
  GList* iter;

  gboolean dispose_has_run;
};


/*    signals enumeration 
 */
enum {
  OPEN_FAIL,                /* file not opened */
  SAVE_FAIL,                /* file not saved  */
  CARD_READ,                /* card read       */ 
  ADDRESSBOOK_READ,         /* file read       */ 
  ADDRESSBOOK_SAVED,        /* file saved      */
  LAST_SIGNAL 
};

static guint rubrica_abook_signals[LAST_SIGNAL] = {0};



static void rubrica_abook_class_init (RubricaAbookClass* klass);
static void rubrica_abook_init       (RubricaAbook* obj);

static void rubrica_abook_dispose    (RubricaAbook* obj);
static void rubrica_abook_finalize   (RubricaAbook* obj);


/*
static void rubrica_abook_set_property (GObject* obj, guint property_id,
					const GValue* value, GParamSpec* spec);
static void rubrica_abook_get_property (GObject* obj, guint property_id,
					GValue* value, GParamSpec* spec);
*/


static gboolean rubrica_abook_open_rubrica_file (RubricaAbook* self, 
						 gchar* filename);
static gboolean rubrica_abook_parse_doc         (RubricaAbook* self,
						 xmlDocPtr xmldoc);

static gboolean rubrica_abook_save_rubrica_file (RubricaAbook* self, 
						 gchar* filename);
static gboolean rubrica_abook_write_file        (RubricaAbook* abook, 
						 gchar* filename);


gboolean opening_old = FALSE;


GType
rubrica_abook_get_type()
{
  static GType rubrica_abook_type = 0;
  
  if (!rubrica_abook_type)
    {
      static const GTypeInfo rubrica_abook_info =
	{
	  sizeof(RubricaAbookClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_abook_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaAbook),
	  0,
	  (GInstanceInitFunc) rubrica_abook_init
	};

      rubrica_abook_type = g_type_register_static (G_TYPE_OBJECT, 
						   "RubricaAbook",
						   &rubrica_abook_info, 0);
    }
  
  return rubrica_abook_type;
}


static void
rubrica_abook_class_init(RubricaAbookClass* klass)
{
  GObjectClass *class;
  //  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose      = (GObjectFinalizeFunc) rubrica_abook_dispose;
  class->finalize     = (GObjectFinalizeFunc) rubrica_abook_finalize;
  //  class->set_property = rubrica_abook_set_property;
  //  class->get_property = rubrica_abook_get_property;


  klass->open_file = rubrica_abook_open_rubrica_file;
  klass->save_file = rubrica_abook_save_rubrica_file;
  klass->export    = NULL;


  /* class property 
     pspec = g_param_spec_boolean("have-name",
			       "have-name",
			       "has got the addressbook name", 
			       FALSE,  
			       G_PARAM_READABLE | G_PARAM_WRITABLE);
  g_object_class_install_property(class, RUBRICA_ABOOK_HAVE_NAME, pspec);

  pspec = g_param_spec_string("name",
			      "name",
			      "addressbook name (filename)",
			      "no name", 
			      G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
  g_object_class_install_property(class, RUBRICA_ABOOK_NAME, pspec);

  pspec = g_param_spec_string("path",
			      "path",
			      "addressbook path (file path)",
			      NULL,
			      G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
  g_object_class_install_property(class, RUBRICA_ABOOK_PATH, pspec);
*/

  
  /* class signals */
  rubrica_abook_signals[OPEN_FAIL] =
    g_signal_new("open_fail", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, abook_open_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */

  rubrica_abook_signals[SAVE_FAIL] =
    g_signal_new("save_fail", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, abook_save_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */


  rubrica_abook_signals[CARD_READ] = 
    g_signal_new("card_read", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, abook_item_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  
  rubrica_abook_signals[ADDRESSBOOK_READ] =
    g_signal_new("addressbook_read", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, abook_addressbook_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);


  rubrica_abook_signals[ADDRESSBOOK_SAVED] =
    g_signal_new("addressbook_saved", 
		 RUBRICA_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaAbookClass, abook_addressbook_saved),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);
}


static void
rubrica_abook_init(RubricaAbook* self)
{
  self->private = g_new(RubricaAbookPrivate, 1);

  self->private->have_name = FALSE;
  self->private->name      = g_strdup(gettext("no name"));
  self->private->path      = NULL;
  
  self->private->cards     = NULL;
  self->private->iter      = NULL;

  self->private->dispose_has_run = FALSE;
}

/* 

static void
rubrica_abook_set_property (GObject* obj, guint property_id,
			    const GValue* value, GParamSpec* spec)
{
  RubricaAbook* self = (RubricaAbook*) obj;

  switch (property_id)
    {
    case RUBRICA_ABOOK_HAVE_NAME:
      self->private->have_name = g_value_get_boolean(value);
      break;

    case RUBRICA_ABOOK_NAME:
      g_free(self->private->name);
      self->private->name = g_value_dup_string(value);
      break;

    case RUBRICA_ABOOK_PATH:
      g_free(self->private->path);
      self->private->path = g_value_dup_string(value);      
      break;
      
    default:
      break;
    }
}


static void
rubrica_abook_get_property (GObject* obj, guint property_id,
			    GValue* value, GParamSpec* spec)
{
  RubricaAbook* self = (RubricaAbook*) obj;

  switch (property_id)
    {
    case RUBRICA_ABOOK_HAVE_NAME:
      g_value_set_boolean(value, self->private->have_name);
      break;
      
    case RUBRICA_ABOOK_NAME:
      g_value_set_string(value, self->private->name);
      break;

    case RUBRICA_ABOOK_PATH:
      g_value_set_string(value, self->private->path);
      break;
      
    default:
      g_assert(FALSE);
      break;
    }  
}

*/

static void 
rubrica_abook_dispose (RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
  
  /* free the list's items (PersonalCard or CompanyCard) 
   */
  self->private->iter = self->private->cards;
  for (; self->private->iter; self->private->iter = self->private->iter->next)
    {
      RubricaCard* card = self->private->iter->data;

      if (IS_RUBRICA_CARD(card))
	rubrica_card_free(RUBRICA_CARD(card));
    }
  
  g_list_free(self->private->cards);
  self->private->cards = NULL;
  self->private->iter  = NULL;
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_abook_finalize (RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
  
  rubrica_utils_free_string(self->private->name);
  rubrica_utils_free_string(self->private->path);

  g_free(self->private);
  self->private = NULL;
}


static gboolean 
rubrica_abook_open_rubrica_file (RubricaAbook* self, gchar* filename) 
{ 
  xmlDocPtr xmldoc = NULL;
  RubricaError err; 
  gboolean loaded; 
  gint fileformat  = 0;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  
  if (!filename)
    {
      g_signal_emit_by_name(self, "open_fail", NO_FILENAME, G_TYPE_INT);

      return FALSE;
    }

  if (!g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      g_signal_emit_by_name(self, "open_fail", FILE_NOT_EXIST, G_TYPE_INT);

      return FALSE;
    }

  if((xmldoc = rubrica_load_open_doc(filename, &err)) == NULL)
    {
      g_signal_emit_by_name(self, "open_fail", err, G_TYPE_INT);

      return FALSE;
    }

  if(!rubrica_load_is_rubrica_doc(xmldoc, &err))
    {
      g_signal_emit_by_name(self, "open_fail", err, G_TYPE_INT);
      
      xmlFreeDoc(xmldoc);
      return FALSE;
    }

  /* get the file format
   */
  fileformat = rubrica_load_get_fileformat(xmldoc, &err);
  if (fileformat < RUBRICA_FILE_FORMAT)  /* this is an old rubrica file */
    {
      /* close opened file 
      xmlFreeDoc(xmldoc);
      */
      if (fileformat == 3)
	{
	  /* error signals emitted in rubrica_xcard_load_ff3_doc
	  if (!rubrica_xcard_load_ff3_doc(xcard, fname))
	    return FALSE;	  
	  else
	    return TRUE;
	  */
	}      
      /*
      if (!rubrica_xcard_io_import_old_file(filename))
	return FALSE;
      */
      /*
      if((xmldoc = rubrica_io_open_doc(filename, &err)) == NULL)
	return FALSE;
      
      if(!rubrica_xio_is_rubrica_doc(xmldoc, &err))
	return NULL;     
      */
    }

  loaded = rubrica_abook_parse_doc(self, xmldoc);

  if (!loaded)
    {
      g_signal_emit_by_name(self, "open_fail", FILE_NOT_OPENED, G_TYPE_INT);
      
      xmlFreeDoc(xmldoc);

      return FALSE;
    }
  else
    g_signal_emit_by_name(self, "addressbook_read", NULL, G_TYPE_NONE);

  xmlFreeDoc(xmldoc);

  rubrica_abook_set_name(self, g_path_get_basename(filename));
  rubrica_abook_set_path(self, g_path_get_dirname(filename));
  
  return TRUE;
}


static gboolean 
rubrica_abook_parse_doc(RubricaAbook* self, xmlDocPtr xmldoc)
{  
  xmlNodePtr xmlabook;
  RubricaError err;
     
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  g_return_val_if_fail(xmldoc != NULL, FALSE);
  
  xmlabook = xmldoc->children;
  if (xmlIsBlankNode(xmlabook))
    xmlabook = xmlabook->next;
  
  if (xmlabook && xmlabook->name && xmlStrcmp(xmlabook->name, "Rubrica") == 0) 
    {
      xmlabook = xmlabook->children;
      
      if (xmlIsBlankNode(xmlabook))
	xmlabook = xmlabook->next;

      while (xmlabook)
	{
	  xmlNodePtr xmlcard = NULL;

	  xmlcard = rubrica_io_get_node(xmlabook, "Card");
	  
	  if (xmlcard)
	    {
	      gchar* tmp = NULL;
	      
	      tmp = rubrica_io_get_prop(xmlcard, "type", &err);
	      if (!tmp)
		tmp = g_strdup("personal");
	      if (g_ascii_strcasecmp("personal", tmp) == 0)
		{
		  RubricaPersonalCard *card;  

		  card = rubrica_load_read_personal_xmlcard(xmlcard);  
		  if (card)
		    {
		      rubrica_abook_add_card(self, (gpointer) card); 
		      g_signal_emit_by_name(self, "card_read", card, 
					    G_TYPE_POINTER);
		    }
		}
	      else if (g_ascii_strcasecmp("company", tmp) == 0)
		{
		  RubricaCompanyCard* card;

		  card = rubrica_load_read_company_xmlcard(xmlcard);

		  if (card)
		    {
		      rubrica_abook_add_card(self, (gpointer) card);
		      
		      g_signal_emit_by_name(self, "card_read", card, 
					    G_TYPE_POINTER);
		    }
		}
	      
	      g_free(tmp);	   	      	      	     
	    }  

	   /* parse next Card */
	   xmlabook = xmlabook->next; 
	   if(xmlIsBlankNode(xmlabook))
	     xmlabook = xmlabook->next;
	}
    }
  else
    {
      g_signal_emit_by_name(self, "open_fail", NO_RUBRICA_DOC, G_TYPE_INT);
      
      return FALSE;
    }

  return TRUE;
}



static gboolean 
rubrica_abook_save_rubrica_file (RubricaAbook* self, gchar* filename)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
    
  if (!filename)
    {
      g_signal_emit_by_name(self, "save_fail", NO_FILENAME, G_TYPE_INT);

      return FALSE;
    }

  if (g_file_test(filename, G_FILE_TEST_EXISTS))
    {
      g_signal_emit_by_name(self, "save_fail", FILE_EXIST, G_TYPE_INT);

      return FALSE;
    }

  if (!rubrica_abook_write_file(self, filename))
    {
      g_signal_emit_by_name(self, "save_fail", WRITING_XML, G_TYPE_INT);
      
      return FALSE;
    }
    
  g_signal_emit_by_name(self, "addressbook_saved", NULL, G_TYPE_NONE);
 
  return TRUE;
}


static gboolean
rubrica_abook_write_file(RubricaAbook* self, gchar* name)
{
  xmlDocPtr doc;        /* puntatore al documento xml */
  gpointer card;
  gchar *path = NULL, *fname;
  gchar *fileformat;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);
  g_return_val_if_fail(name != NULL, FALSE);

  path  = g_dirname(name);
  fname = g_strdup(name);

  /* creazione di un nuovo documento xml 
   */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica 
   */
  fileformat = g_strdup_printf("%d", RUBRICA_FILE_FORMAT);

  doc->children = xmlNewDocNode(doc, NULL, "Rubrica", NULL);
  xmlSetProp(doc->children, "version", (xmlChar*) RUBRICA_VERSION);
  xmlSetProp(doc->children, "fileformat", fileformat); 
  xmlSetProp(doc->children, "doctype", "AddressBook"); 

  g_free(fileformat);

  /* attraverso la rubrica e scrivo i dati 
     nell'albero xml
  */
  rubrica_abook_reset_book(self);
  card = rubrica_abook_get_card(self);
  for ( ; card; card = rubrica_abook_get_next_card(self))
    {
      RubricaCardClass* klass;
      xmlNodePtr cardxml;     /* card node */

      klass = RUBRICA_CARD_GET_CLASS(card);

      if (RUBRICA_CARD_CLASS(klass)->write_node)
	{
	  cardxml = xmlNewChild(doc->children, NULL, "Card", NULL);

	  rubrica_save_write_card(cardxml, RUBRICA_CARD(card));
	  RUBRICA_CARD_CLASS(klass)->write_node(cardxml, (gpointer) card);
	}  
    }     
  
  if (xmlSaveFormatFile(fname, doc, 1) == -1) 
    return FALSE;

  /* if we are here the addressbook is write 
   */  

  /* xmlFreeDoc do a segfault, why? */
  xmlFreeDoc(doc);    
  g_free(fname);

  return TRUE;
}



/*   Public
*/
RubricaAbook*
rubrica_abook_new(void)
{
  RubricaAbook* self;

  self = g_object_new(rubrica_abook_get_type(), NULL);

  return self;
}


void
rubrica_abook_free(RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));

  g_object_unref(self);   
}



gboolean 
rubrica_abook_open_file (RubricaAbook* self, gchar* filename)
{
  RubricaAbookClass* class;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);

  class = RUBRICA_ABOOK_GET_CLASS(self);
  
  if (class->open_file)
    return class->open_file(self, filename); 

  return FALSE;
}



gboolean 
rubrica_abook_save_file (RubricaAbook* self, gchar* filename)
{
  RubricaAbookClass* class;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), FALSE);

  class = RUBRICA_ABOOK_GET_CLASS(self);
  
  if (class->save_file)
    return class->save_file(self, filename); 
  
  return FALSE;
}



RubricaAbook* 
rubrica_abook_copy (RubricaAbook* self)
{
  RubricaAbook* new;
  gpointer card;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);
  
  new = rubrica_abook_new();
  
  card = rubrica_abook_get_card(self);
  for (; card; card = rubrica_abook_get_next_card(self))
    {
      gpointer new_card = NULL; 

      new_card = rubrica_card_copy(RUBRICA_CARD(card));
      
      if (new_card)
	rubrica_abook_add_card(new, new_card);
    }    
  rubrica_abook_reset_book(self);
  
  return new;
}



void   
rubrica_abook_set_name (RubricaAbook* self, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));

  rubrica_utils_free_string(self->private->name);
  self->private->name = g_strdup(name);
}



void   
rubrica_abook_set_path (RubricaAbook* self, gchar* path)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));

  rubrica_utils_free_string(self->private->path);
  self->private->path = g_strdup(path);  
}



gchar* 
rubrica_abook_get_name (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);

  return self->private->name;  
}



gchar* 
rubrica_abook_get_path (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);
  
  return self->private->path;   
}



gint
rubrica_abook_get_items (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), 0);

  return g_list_length(self->private->cards);
}



void   
rubrica_abook_add_card (RubricaAbook* self, gpointer card)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
 
  self->private->cards = g_list_append(self->private->cards, card);
  self->private->iter  = self->private->cards;
}



void   
rubrica_abook_remove_card (RubricaAbook* self, gpointer card)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
  g_return_if_fail(card != NULL);
  
  self->private->cards = g_list_remove(self->private->cards, card);
  self->private->iter  = self->private->cards;
  
  // TODO: push card in undo_list
}



void   
rubrica_abook_modify_card (RubricaAbook* self, gpointer card)
{
  gint pos;
  GList* node;
  gpointer old;

  g_return_if_fail(IS_RUBRICA_ABOOK(self));
      
  pos = g_list_index(self->private->cards, card);
  node = g_list_nth(self->private->cards, pos);
  if (node)
    {
      old = node->data;
      rubrica_card_free(RUBRICA_CARD(old));
    }

  g_list_insert(self->private->cards, card, pos);
}



gpointer 
rubrica_abook_get_card (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);

  if (self->private->iter)
    return self->private->iter->data;

  return NULL;
}



gpointer      
rubrica_abook_get_card_by_id (RubricaAbook* self, gchar* id)
{
  gpointer card = NULL;

  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);
  g_return_val_if_fail(id != NULL, NULL);
  
  for (card = rubrica_abook_get_card(self); 
       card; 
       card = rubrica_abook_get_next_card(self))
    {
      gchar* card_id;

      card_id = rubrica_card_get_id(RUBRICA_CARD(card));
      if (g_ascii_strcasecmp(id, card_id) == 0)
	return card;      
    }  
  rubrica_abook_reset_book(self);

  return NULL;
}



void     
rubrica_abook_reset_book (RubricaAbook* self)
{
  g_return_if_fail(IS_RUBRICA_ABOOK(self));
 
  self->private->iter = self->private->cards;
}


gpointer
rubrica_abook_get_next_card (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);
 
  self->private->iter = g_list_next(self->private->iter);
  if (self->private->iter)
    return self->private->iter->data;
  else
    self->private->iter = self->private->cards;

  return NULL;
}


gpointer 
rubrica_abook_get_prev_card (RubricaAbook* self)
{
  g_return_val_if_fail(IS_RUBRICA_ABOOK(self), NULL);
 
  self->private->iter = g_list_previous(self->private->iter);
  if (self->private->iter)
    return self->private->iter->data;
  else self->private->iter = self->private->cards;
  
  return NULL;
}
