/*
*  RAL -- Rubrica Addressbook Library
*  file: address.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "address.h"
#include "date.h"
#include "utils.h"


struct _RubricaAddressPrivate {
  gboolean dispose_has_run;
};


typedef struct {
  gint   token;
  gchar* label;
} AddressConversion;


static AddressConversion table[] = {
  {RUBRICA_ADDRESS_PREF,    "pref"   },
  {RUBRICA_ADDRESS_HOME,    "home"   },
  {RUBRICA_ADDRESS_WORK,    "work"   },
  {RUBRICA_ADDRESS_PARCEL,  "parcel" },
  {RUBRICA_ADDRESS_POSTAL,  "postal" },
  {RUBRICA_ADDRESS_INTL,    "intl"   },
  {RUBRICA_ADDRESS_DOM,     "dom"    },
  {RUBRICA_ADDRESS_COMPANY, "company"},
  {RUBRICA_ADDRESS_UNKNOWN, "unknown"},
  {RUBRICA_ADDRESS_INVALID, "invalid"},
};


static void rubrica_address_class_init (RubricaAddressClass* klass);
static void rubrica_address_init       (RubricaAddress* obj);

static void rubrica_address_dispose    (RubricaAddress* obj);
static void rubrica_address_finalize   (RubricaAddress* obj);


GType
rubrica_address_get_type()
{
  static GType address_type = 0;
  
  if (!address_type)
    {
      static const GTypeInfo address_info =
	{
	  sizeof(RubricaAddressClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_address_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaAddress),
	  0,
	  (GInstanceInitFunc) rubrica_address_init
	};

      address_type = g_type_register_static (G_TYPE_OBJECT, "RubricaAddress",
					     &address_info, 0);
    }
  
  return address_type;
}


static void
rubrica_address_class_init(RubricaAddressClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_address_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_address_finalize;
}


static void
rubrica_address_init(RubricaAddress* self)
{
  self->type     = NULL;
  self->street   = NULL;
  self->number   = NULL;
  self->city     = NULL;
  self->zip      = NULL;
  self->province = NULL;
  self->state    = NULL;
  self->country  = NULL;

  self->private = g_new(RubricaAddressPrivate, 1);
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_address_dispose (RubricaAddress* self)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_address_finalize (RubricaAddress* self)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->type);
  rubrica_utils_free_string(self->street);
  rubrica_utils_free_string(self->number);
  rubrica_utils_free_string(self->city);
  rubrica_utils_free_string(self->zip);
  rubrica_utils_free_string(self->province);
  rubrica_utils_free_string(self->state);
  rubrica_utils_free_string(self->country);

  g_free(self->private);
  self->private = NULL;
}


RubricaAddress*
rubrica_address_new(void)
{
  RubricaAddress* self;

  self = g_object_new(rubrica_address_get_type(), NULL);

  return self;
}


void
rubrica_address_free(RubricaAddress* self)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  g_object_unref(self);
}



void 
rubrica_address_set (RubricaAddress* self, RubricaAddressType type,
		     gchar* street, gchar* number, 
		     gchar* city, gchar* zip, gchar* province, 
		     gchar* state, gchar* country)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_address_set_address_type (self, type);
  rubrica_address_set_street (self, street);
  rubrica_address_set_number (self, number);
  rubrica_address_set_city   (self, city);
  rubrica_address_set_zip    (self, zip);
  rubrica_address_set_province (self, province);
  rubrica_address_set_state (self, state);
  rubrica_address_set_country (self,country);
}


void
rubrica_address_set_address_type (RubricaAddress* self,
				  RubricaAddressType type)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->type);
  self->type = g_strdup(table[type].label);
}

void 
rubrica_address_set_street (RubricaAddress* self, gchar* street)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->street);
  self->street = g_strdup(street);
}


void 
rubrica_address_set_number (RubricaAddress* self, gchar* number)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->number);
  self->number = g_strdup(number);
}


void 
rubrica_address_set_city (RubricaAddress* self, gchar* city)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->city);
  self->city = g_strdup(city);
}


void 
rubrica_address_set_zip (RubricaAddress* self, gchar* zip)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->zip);
  self->zip = g_strdup(zip);
}


void 
rubrica_address_set_province (RubricaAddress* self, gchar* province)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));
  
  rubrica_utils_free_string(self->province);
  self->province = g_strdup(province);
}


void 
rubrica_address_set_state (RubricaAddress* self, gchar* state)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->state);
  self->state = g_strdup(state);
}


void 
rubrica_address_set_country (RubricaAddress* self, gchar* country)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  rubrica_utils_free_string(self->country);
  self->country = g_strdup(country);
}


void 
rubrica_address_get (RubricaAddress* self, gchar **type,
		     gchar** street, gchar** number, 
		     gchar** city, gchar** zip, gchar** province, 
		     gchar** state, gchar** country)
{
  g_return_if_fail(IS_RUBRICA_ADDRESS(self));

  if (type)
    *type = rubrica_address_get_address_type(self);
  if (street)
    *street = rubrica_address_get_street(self);
  if (number)
    *number = rubrica_address_get_number(self);
  if (city)
    *city = rubrica_address_get_city(self);
  if (zip)
    *zip = rubrica_address_get_zip(self);
  if (province)
    *province = rubrica_address_get_province(self);
  if (state)
    *state = rubrica_address_get_state(self); 
  if (country)
    *country = rubrica_address_get_country(self);
}

gchar* 
rubrica_address_get_address_type (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);

  return self->type;
}


gchar* 
rubrica_address_get_street (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->street;
}

gchar* 
rubrica_address_get_number (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->number;
}


gchar* 
rubrica_address_get_city (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->city;
}


gchar* 
rubrica_address_get_zip (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->zip;
}


gchar* 
rubrica_address_get_province (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->province;
}


gchar* 
rubrica_address_get_state (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->state;
}


gchar* 
rubrica_address_get_country (RubricaAddress* self)
{
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);
  
  return self->country;
}


RubricaAddress* 
rubrica_address_copy  (RubricaAddress* self)
{
  RubricaAddress* new;
  gchar *tmp;
  
  g_return_val_if_fail(IS_RUBRICA_ADDRESS(self), NULL);

  new = rubrica_address_new();
    
  tmp = rubrica_address_get_address_type(self);
  rubrica_address_set(new, 
		      rubrica_address_get_decoded_type(tmp),
		      rubrica_address_get_street(self),
		      rubrica_address_get_number(self),
		      rubrica_address_get_city(self),
		      rubrica_address_get_zip(self),
		      rubrica_address_get_province(self),
		      rubrica_address_get_state(self),
		      rubrica_address_get_country(self));

  return new;
}


RubricaAddressType 
rubrica_address_get_decoded_type (gchar* type)
{
  RubricaAddressType ret = RUBRICA_ADDRESS_INVALID;

  if (!type)
    return ret;

  if (g_ascii_strcasecmp(type, "pref") == 0)
    ret = RUBRICA_ADDRESS_PREF;
  else if (g_ascii_strcasecmp(type, "home") == 0)
    ret = RUBRICA_ADDRESS_HOME;
  else if (g_ascii_strcasecmp(type, "work") == 0)
    ret = RUBRICA_ADDRESS_WORK;
  else if (g_ascii_strcasecmp(type, "parcel") == 0)
    ret = RUBRICA_ADDRESS_PARCEL;
  else if (g_ascii_strcasecmp(type, "postal") == 0)
    ret = RUBRICA_ADDRESS_POSTAL;
  else if (g_ascii_strcasecmp(type, "intl") == 0)
    ret = RUBRICA_ADDRESS_INTL;
  else if (g_ascii_strcasecmp(type, "dom") == 0)
    ret = RUBRICA_ADDRESS_DOM;
  else if (g_ascii_strcasecmp(type, "company") == 0)
    ret = RUBRICA_ADDRESS_COMPANY;
  else
    ret = RUBRICA_ADDRESS_UNKNOWN;
 
  return ret;  
}
