/*
*  RAL -- Rubrica Addressbook Library
*  file: card.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <string.h>
#include <glib.h>
#include <glib-object.h>
#include <libxml/tree.h>

#include "card.h"
#include "groups.h"
#include "utils.h"


gulong personal_id = 0;
gulong company_id  = 0;


struct _RubricaCardPrivate {
  gchar* id;                 /* card's unique id         */
  gchar* type;               /* card's type              */

  time_t created;            /* card's creation time     */
  time_t changed;            /* card's last change time  */

  GList* addr_iter;          /* addresses, telephone, nets iterators */
  GList* net_iter;
  GList* phone_iter;

  gboolean dispose_has_run;
};


static void rubrica_card_class_init (RubricaCardClass* klass);
static void rubrica_card_init       (RubricaCard* self);

static void rubrica_card_dispose    (RubricaCard* self);
static void rubrica_card_finalize   (RubricaCard* self);


static gulong rubrica_card_generate_id (RubricaCard* card, gchar* type);
static void   rubrica_card_align_id    (RubricaCard* card, gchar* id, 
					gchar* type);
static void   rubrica_card_real_copy   (RubricaCard* new, RubricaCard* old);


/*  Private functions
*/
GType
rubrica_card_get_type()
{
  static GType rubrica_card_type = 0;
  
  if (!rubrica_card_type)
    {
      static const GTypeInfo rubrica_card_info =
	{
	  sizeof(RubricaCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaCard),
	  0,
	  (GInstanceInitFunc) rubrica_card_init
	};

      rubrica_card_type = g_type_register_static (G_TYPE_OBJECT, "RubricaCard",
						  &rubrica_card_info, 0);
    }

  return rubrica_card_type;
}


static void
rubrica_card_class_init(RubricaCardClass* klass)
{
  GObjectClass *class;
  
  klass->free        = NULL;
  klass->generate_id = rubrica_card_generate_id;
  klass->align_id    = rubrica_card_align_id;
  klass->write_node  = NULL;
  klass->copy        = NULL;

  klass->info_type   = NULL;
  klass->print_card  = NULL;

  class = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_card_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_card_finalize;
}


static void
rubrica_card_init(RubricaCard* self)
{
  self->name      = NULL;
  self->rank      = RUBRICA_RANK_NORMAL;
  self->groups    = rubrica_groups_new();
  self->deletable = TRUE;
  self->deleted   = FALSE;
  self->marked    = FALSE;
  self->addresses = NULL;
  self->nets      = NULL;
  self->phones    = NULL;

  self->private = g_new(RubricaCardPrivate, 1);

  self->private->id              = NULL;
  self->private->created         = time(NULL);
  self->private->changed         = time(NULL);
  self->private->addr_iter       = NULL;
  self->private->net_iter        = NULL;
  self->private->phone_iter      = NULL;
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_card_dispose (RubricaCard* self)
{
  //  RubricaCardClass* klass;
  GList* del;  

  g_return_if_fail(IS_RUBRICA_CARD(self));    

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;

  /* free card's children 
  
  klass = RUBRICA_CARD_GET_CLASS(self); 
  RUBRICA_CARD_CLASS(klass)->free(self); 
  */
  RUBRICA_CARD_CLASS(RUBRICA_CARD_GET_CLASS(self))->free(self);

  /* free card's data
   */
  rubrica_utils_free_string(self->name);
  self->name = NULL;

  rubrica_groups_free(self->groups);  
  self->groups = NULL;

  del = self->addresses;
  for (; del; del = del->next)
    {
      RubricaAddress* address = del->data;;

      if(address)
	rubrica_address_free(RUBRICA_ADDRESS(address));
    }
  self->addresses = NULL;

  del = self->nets;
  for (; del; del = del->next)
    {
      RubricaNetAddress* net = del->data;;

      if(net)
	rubrica_net_address_free(RUBRICA_NET_ADDRESS(net));
    }
  self->nets = NULL;

  del = self->phones;
  for (; del; del = del->next)
    {
      RubricaTelephone* tel = del->data;;

      if(tel)
	rubrica_telephone_free(RUBRICA_TELEPHONE(tel));
    }
  self->phones = NULL;   
}


static void 
rubrica_card_finalize (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
 
  rubrica_utils_free_string(self->private->id);
  self->private->id   = NULL;
  self->private->type = NULL;

  self->private->addr_iter  = NULL;
  self->private->net_iter   = NULL;
  self->private->phone_iter = NULL;

  g_free(self->private);
  self->private = NULL;
}


static gulong 
rubrica_card_generate_id (RubricaCard* self, gchar* type) 
{
  gulong ret = 0L;

  g_return_val_if_fail(IS_RUBRICA_CARD(self), 0L); 
  
  if (g_ascii_strcasecmp(type, "personal") == 0)
    {
      self->type = g_strdup("personal");
      ret = ++personal_id;
    }
  else if (g_ascii_strcasecmp(type, "company") == 0)
    {
      self->type = g_strdup("company");
      ret = ++company_id;
    }

  return ret;
}


static void   
rubrica_card_align_id (RubricaCard* self, gchar* id, gchar* type)
{
  gulong cid = 0L;  

  g_return_if_fail(IS_RUBRICA_CARD(self)); 
  g_return_if_fail(id != NULL);
  
  id+=2;       /* sposto il ptr avanti di due byte (pcxxx) */  
  cid = atol(id);
   
  if (g_ascii_strcasecmp(type, "personal") == 0)
    {
      if (cid >= personal_id)
	personal_id = cid;
    }
  else if (g_ascii_strcasecmp(type, "company") == 0)
    {
      if (cid >= company_id)
	company_id = cid;
    }  
}


/**
 * rubrica_card_copy
 *
 * 
 */
static void
rubrica_card_real_copy (RubricaCard* new, RubricaCard* old)
{
  gpointer data;
  RubricaGroups* groups;
  time_t time;

  g_return_if_fail(IS_RUBRICA_CARD(new));
  g_return_if_fail(IS_RUBRICA_CARD(old));
  
  rubrica_card_set(new, 
		   rubrica_card_is_deletable(old),
		   rubrica_card_is_deleted(old),
		   rubrica_card_is_marked(old),
		   rubrica_card_get_card_name(old),
		   rubrica_card_get_rank(old));

  time = rubrica_card_get_creation_time(old);
  rubrica_card_set_creation_time(new, time);

  time = rubrica_card_get_modification_time(old);
  rubrica_card_set_modification_time(new, time);

  groups = rubrica_groups_copy(rubrica_card_get_groups(RUBRICA_CARD(old)));
  rubrica_card_set_groups(new, groups); 
  
  data = rubrica_card_get_address(RUBRICA_CARD(old));
  for  (; data; data = rubrica_card_get_next_address(RUBRICA_CARD(old)))
    {
      RubricaAddress* address;
      
      address = rubrica_address_copy(RUBRICA_ADDRESS(data));
      if (address)
	rubrica_card_add_address(RUBRICA_CARD(new), address);
    }
 
  data = rubrica_card_get_net_address(RUBRICA_CARD(old));
  for (; data; data = rubrica_card_get_next_net_address(RUBRICA_CARD(old)))
    {
      RubricaNetAddress* net;

      net = rubrica_net_address_copy(RUBRICA_NET_ADDRESS(data));
      if (net)
	rubrica_card_add_net_address(RUBRICA_CARD(new), net);
    }
  
  data = rubrica_card_get_telephone(RUBRICA_CARD(old));
  for (; data; data = rubrica_card_get_next_telephone(RUBRICA_CARD(old)))
    {
      RubricaTelephone* tel;

      tel = rubrica_telephone_copy(RUBRICA_TELEPHONE(data));
      if (tel)
	rubrica_card_add_telephone(RUBRICA_CARD(new), tel);
    }
}



/* ************************************  Public Functions
 */

/**
 * rubrica_card_new
 *
 * return a new allocated RubricaCard
 */
RubricaCard*  
rubrica_card_new(void)
{
  RubricaCard* self;

  self = g_object_new(rubrica_card_get_type(), NULL);

  return self;
}


/**
 * rubrica_card_free
 *
 * 
 */
void 
rubrica_card_free(RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  
  g_object_unref(G_OBJECT(self));  
}


gpointer 
rubrica_card_copy (RubricaCard* self)
{
  RubricaCardClass* klass;
  RubricaCard* new;
  
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);   

  klass = RUBRICA_CARD_GET_CLASS(self);

  if (RUBRICA_CARD_CLASS(klass)->copy)
    {
      new = RUBRICA_CARD_CLASS(klass)->copy(G_OBJECT(self));    
      rubrica_card_real_copy(RUBRICA_CARD(new), RUBRICA_CARD(self));

      return (gpointer) new;
    }
  
  return NULL;
}


gint              
rubrica_card_info_type (RubricaCard* self)
{
  RubricaCardClass* klass;
  
  g_return_val_if_fail(IS_RUBRICA_CARD(self), UNKNOW_CARD);   

  klass = RUBRICA_CARD_GET_CLASS(self);

  return RUBRICA_CARD_CLASS(klass)->info_type(G_OBJECT(self));  
}


void 
rubrica_card_print (RubricaCard* self)
{
  RubricaCardClass* klass;
  
  g_return_if_fail(IS_RUBRICA_CARD(self));   

  klass = RUBRICA_CARD_GET_CLASS(self);

  return RUBRICA_CARD_CLASS(klass)->print_card(G_OBJECT(self));    
}


/**
 * rubrica_card_set_id
 *
 * 
 */
void      
rubrica_card_set_id (RubricaCard* self, gchar* id) 
{ 
  g_return_if_fail(IS_RUBRICA_CARD(self));   
  
  rubrica_utils_free_string(self->private->id); 
  self->private->id = g_strdup(id); 
}



/**
 * rubrica_card
 *
 * 
 */
void     
rubrica_card_set (RubricaCard* self,
		  gboolean deletable, gboolean delete, 
		  gboolean marked, gchar* name, 
		  RubricaRank rank)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  rubrica_card_set_deletable(self, deletable);
  rubrica_card_set_deleted(self, delete);
  rubrica_card_set_marked(self, marked);
  rubrica_card_set_card_name(self, name);
  rubrica_card_set_rank(self, rank);
}


/**
 * rubrica_card
 *
 * 
 */
void
rubrica_card_set_card_name (RubricaCard* self, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  rubrica_utils_free_string(self->name);
  self->name = g_strdup(name);
}


/**
 * rubrica_card
 *
 * 
 */
void 
rubrica_card_set_rank (RubricaCard* self, RubricaRank rank)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->rank = rank;
}



/**
 * rubrica_card
 *
 * 
 */
void 
rubrica_card_set_groups (RubricaCard* self, RubricaGroups* groups)
{
  g_return_if_fail(IS_RUBRICA_CARD(self)); 

  rubrica_groups_free(self->groups);
  self->groups = groups;
}



/**
 * rubrica_card_set_deleted
 *
 * 
 */
void 
rubrica_card_set_deleted(RubricaCard *self, gboolean state)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->deleted = (gboolean) state;
}


/**
 * rubrica_card_set_deletable
 *
 * 
 */
void 
rubrica_card_set_deletable (RubricaCard* self, gboolean bool)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->deletable = bool;  
}


/**
 * rubrica_card_set_marked
 *
 * 
 */
void 
rubrica_card_set_marked (RubricaCard* self, gboolean bool)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->marked = bool;
}


/**
 * rubrica_card_creation_time
 *
 * 
*/
void  
rubrica_card_set_creation_time (RubricaCard* self, time_t time) 
{ 
  g_return_if_fail(IS_RUBRICA_CARD(self)); 

  self->private->created = time; 
}



/**
 * rubrica_card_set_modification_time
 *
 * 
 */
void 
rubrica_card_set_modification_time (RubricaCard* self, time_t time)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->private->changed = time;
}


/**
 * rubrica_card_add_group
 *
 * 
 */
void 
rubrica_card_add_group (RubricaCard* self, gchar* group)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  g_return_if_fail(group != NULL);

  rubrica_groups_add_group(self->groups, group);
}


/**
 * rubrica_card
 *
 * 
 */
void 
rubrica_card_add_address (RubricaCard* self, RubricaAddress* address)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  g_return_if_fail(IS_RUBRICA_ADDRESS(address));

  self->addresses = g_list_append(self->addresses, address);

  /* init address iterator */
  if (!self->private->addr_iter)
    self->private->addr_iter = self->addresses;
}


/**
 * rubrica_card_add_net_address 
 *
 * 
 */
void 
rubrica_card_add_net_address (RubricaCard* self, RubricaNetAddress* netadd)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(netadd));

  self->nets = g_list_append(self->nets, netadd);

  /* init card iterator */
  if (!self->private->net_iter)
    self->private->net_iter = self->nets;
}


/**
 * rubrica_card_add_telephone
 *
 * 
 */
void 
rubrica_card_add_telephone (RubricaCard* self, RubricaTelephone* tel)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  g_return_if_fail(IS_RUBRICA_TELEPHONE(tel));

  self->phones = g_list_append(self->phones, tel);

  /* init telephone iterator */
  if (!self->private->phone_iter)
    self->private->phone_iter = self->phones;
}


/**
 * rubrica_card_get_card_name
 *
 * 
 */
gchar* 
rubrica_card_get_card_name (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  return self->name;
}


/**
 * rubrica_card_get_card_type
 *
 * 
 */
gchar*             
rubrica_card_get_card_type (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  return self->type;
}



/**
 * rubrica_card_get_id
 *
 * 
 */
gchar*
rubrica_card_get_id (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  return self->private->id;
}



/**
 * rubrica_card_get_rank
 *
 * 
 */
RubricaRank
rubrica_card_get_rank (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), RUBRICA_RANK_NORMAL);
    
  return self->rank;
}


/**
 * rubrica_card_belong_to_group
 *
 * 
 */
gboolean
rubrica_card_belong_to_group (RubricaCard* self, gchar* group)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), FALSE); 
    
  return rubrica_groups_owns_group(self->groups, group);
}


/**
 * rubrica_card_get_groups
 *
 * 
 */
RubricaGroups* 
rubrica_card_get_groups (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  return self->groups;
}



/**
 * rubrica_card_is_deletable
 *
 * 
 */
gboolean 
rubrica_card_is_deletable (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), FALSE);
  
  return (gboolean) self->deletable;
}


/**
 * rubrica_card_is_deleted
 *
 * 
 */
gboolean 
rubrica_card_is_deleted (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), FALSE);

  return (gboolean) self->deleted;
}


/**
 * rubrica_card_is_marked
 *
 * 
 */
gboolean 
rubrica_card_is_marked (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), FALSE);

  return (gboolean) self->marked;
}


/**
 * rubrica_card_get_creation_time
 *
 * 
 */
time_t   
rubrica_card_get_creation_time (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), -1);

  return self->private->created;
}


/**
 * rubrica_card_get_modification_time 
 *
 * 
 */
time_t   
rubrica_card_get_modification_time (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), -1);

  return self->private->changed;
}


/**
 * rubrica_card_get_group
 *
 * 
 */
gpointer 
rubrica_card_get_group (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
 
  return rubrica_groups_get_group(self->groups);
}



/**
 * rubrica_card_get_next_group
 *
 * 
 */
gpointer
rubrica_card_get_next_group (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self),NULL);

  return rubrica_groups_get_next_group(self->groups);
}


/**
 * rubrica_card_get_prev_group
 *
 * 
 */
gpointer
rubrica_card_get_prev_group (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  return rubrica_groups_get_prev_group(self->groups);
}



/**
 * rubrica_card_reset_group
 *
 * 
 */
void 
rubrica_card_reset_group (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  rubrica_groups_reset(self->groups);
}



/**
 * rubrica_card_get_address
 *
 * 
 */
gpointer
rubrica_card_get_address (RubricaCard* self)
{
  RubricaAddress* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  if (self->private->addr_iter)
    ret = self->private->addr_iter->data;
  if (IS_RUBRICA_ADDRESS(ret))
    return ret;
  
  return NULL;
}


/**
 * rubrica_card_get_next_address
 *
 * 
 */
gpointer 
rubrica_card_get_next_address (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->addr_iter = g_list_next(self->private->addr_iter);
  if(self->private->addr_iter)
    return self->private->addr_iter->data;
  else  /* iterator is NULL, reset iterator */
    self->private->addr_iter = self->addresses;

  return NULL;
}


/**
 * rubrica_card_get_prev_address
 *
 * 
 */
gpointer
rubrica_card_get_prev_address (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->addr_iter = g_list_previous(self->private->addr_iter);
  if (self->private->addr_iter)
    return self->private->addr_iter->data;
  else  /* iterator is NULL, reset iterator */
    self->private->addr_iter = self->addresses;

  return NULL;
}


/**
 * rubrica_card_reset_address
 *
 * 
 */
void 
rubrica_card_reset_address (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));

  self->private->addr_iter = self->addresses;
}


/**
 * rubrica_card_get_net_address
 *
 * 
 */
gpointer
rubrica_card_get_net_address (RubricaCard* self)
{
  RubricaNetAddress* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  if (self->private->net_iter)
    ret = self->private->net_iter->data;
  if (IS_RUBRICA_NET_ADDRESS(ret))
    return ret;
  
  return NULL;
}


/**
 * rubrica_card_get_next_net_address
 *
 * 
 */
gpointer
rubrica_card_get_next_net_address (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->net_iter = g_list_next(self->private->net_iter);
  if (self->private->net_iter) 
    return self->private->net_iter->data;
  else  /* iterator is NULL, reset iterator */
    self->private->net_iter = self->nets;

  return NULL;  
}


/**
 * rubrica_card_get_prev_net_address
 *
 * 
 */
gpointer
rubrica_card_get_prev_net_address (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->net_iter = g_list_previous(self->private->net_iter);
  if (self->private->net_iter)
    return self->private->net_iter->data;
  else
    self->private->net_iter = self->nets;

  return NULL;  
}


/**
 * rubrica_card_reset_net_address
 *
 * 
 */
void 
rubrica_card_reset_net_address (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  
  self->private->net_iter = self->nets;
}



/**
 * rubrica_card_get_telephone
 *
 * 
 */
gpointer
rubrica_card_get_telephone (RubricaCard* self)
{
  RubricaTelephone* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);

  if (self->private->phone_iter)
    ret = self->private->phone_iter->data;
  if (IS_RUBRICA_TELEPHONE(ret))
    return ret;
  
  return NULL;  
}



/**
 * rubrica_card_get_next_telephone
 *
 * 
 */
gpointer
rubrica_card_get_next_telephone (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->phone_iter = g_list_next(self->private->phone_iter);
  if (self->private->phone_iter)
    return self->private->phone_iter->data;
  else  /* iterator is NULL, reset iterator */
    self->private->phone_iter = self->phones;
  
  return NULL;  
}


/**
 * rubrica_card_get_prev_telephone
 *
 * 
 */
gpointer
rubrica_card_get_prev_telephone (RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(self), NULL);
  
  self->private->phone_iter= g_list_previous(self->private->phone_iter);
  if (self->private->phone_iter)
    return self->private->phone_iter->data;
  else
    self->private->phone_iter = self->phones;

  return NULL;  
}



/**
 * rubrica_card_reset_telephone
 *
 * 
 */
void 
rubrica_card_reset_telephone (RubricaCard* self)
{
  g_return_if_fail(IS_RUBRICA_CARD(self));
  
  self->private->phone_iter = self->phones;
}



/**
 * rubrica_card_need_id
 *
 * return TRUE if card needs an id FALSE otherwise 

gboolean 
rubrica_card_need_id(RubricaCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_CARD(card), FALSE);
  
  return (gboolean) (NULL == self->id);
}
 */


/* eof */
