/*
*  RAL -- Rubrica Addressbook Library
*  file: card.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_CARD_H
#define _RUBRICA_CARD_H

#include <time.h>
#include <glib.h>
#include <glib-object.h>
#include <libxml/tree.h>

#include "groups.h"
#include "address.h"
#include "net.h"
#include "telephone.h"



#define RUBRICA_CARD_TYPE            (rubrica_card_get_type())

#define RUBRICA_CARD(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),   \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCard))

#define RUBRICA_CARD_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),    \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCardClass))

#define IS_RUBRICA_CARD(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj),   \
                                      RUBRICA_CARD_TYPE))

#define IS_RUBRICA_CARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),    \
                                      RUBRICA_CARD_TYPE))

#define RUBRICA_CARD_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),    \
                                      RUBRICA_CARD_TYPE,                  \
                                      RubricaCardClass))


typedef enum {
  RUBRICA_RANK_NONE = 0,         /* no importance  */
  RUBRICA_RANK_FEW,              /* few importance  */
  RUBRICA_RANK_NORMAL,           /* normal  */
  RUBRICA_RANK_HIGHT,            /* hight importance  */
  RUBRICA_RANK_VERY_HIGHT        /* very hight importance  */
} RubricaRank;


enum {
  UNKNOW_CARD,
  PERSONAL_CARD,
  COMPANY_CARD
};


typedef struct _RubricaCard          RubricaCard;
typedef struct _RubricaCardClass     RubricaCardClass;
typedef struct _RubricaCardPrivate   RubricaCardPrivate;


struct _RubricaCard 
{
  GObject parent;
  
  gchar*         name;         /* card's name   */
  gchar*         type;         /* card's type   */
  RubricaRank    rank;         /* card's rank   */
  RubricaGroups* groups;        /* card's groups */

  gboolean       deletable;    /* true if card is marked deletable */
  gboolean       deleted;      /* is this a deleted card ?         */
  gboolean       marked;       /* true if card was marked          */

  GList*         addresses;
  GList*         nets;         /* web, email and irc uries */
  GList*         phones;

  RubricaCardPrivate* private;
};


struct _RubricaCardClass
{
  GObjectClass parent;
  
  /* methods */
  void     (*free)        (GObject* self);
  gint     (*generate_id) (RubricaCard* self, gchar* type);
  void     (*align_id)    (RubricaCard* self, gchar* id, gchar* type);
  void     (*write_node)  (RubricaCard* self, xmlNodePtr node);
  gpointer (*copy)        (RubricaCard* self); 

  /* used for tests, to be removed */
  gint     (*info_type)   (RubricaCard* self);
  void     (*print_card)  (RubricaCard* self); 

  /* signals */
};



GType        rubrica_card_get_type (void);

RubricaCard* rubrica_card_new      (void);
void         rubrica_card_free     (RubricaCard* self);
gpointer     rubrica_card_copy     (RubricaCard* self);


/*  Set
*/
void rubrica_card_set                   (RubricaCard* self,
					 gboolean deletable, gboolean delete, 
					 gboolean marked,    gchar *name,
					 RubricaRank rank);

void rubrica_card_set_id                (RubricaCard* self, gchar* id);

void rubrica_card_set_deletable         (RubricaCard* self, gboolean bool);
void rubrica_card_set_deleted           (RubricaCard* self, gboolean bool);
void rubrica_card_set_marked            (RubricaCard* self, gboolean bool);
void rubrica_card_set_creation_time     (RubricaCard* self, time_t t);
void rubrica_card_set_modification_time (RubricaCard* self, time_t t);
void rubrica_card_set_card_name         (RubricaCard* self, gchar* name);
void rubrica_card_set_rank              (RubricaCard* self, RubricaRank rank);
void rubrica_card_set_groups            (RubricaCard* self, 
					 RubricaGroups* groups);



/*  Get
*/
gchar*         rubrica_card_get_id                (RubricaCard* self); 
gchar*         rubrica_card_get_card_name         (RubricaCard* self);
gchar*         rubrica_card_get_card_type         (RubricaCard* self);
RubricaRank    rubrica_card_get_rank              (RubricaCard* self);

gboolean       rubrica_card_is_deletable          (RubricaCard* self);
gboolean       rubrica_card_is_deleted            (RubricaCard* self);
gboolean       rubrica_card_is_marked             (RubricaCard* self);
time_t         rubrica_card_get_creation_time     (RubricaCard* self);
time_t         rubrica_card_get_modification_time (RubricaCard* self);
RubricaGroups* rubrica_card_get_groups            (RubricaCard* self);


gboolean rubrica_card_belong_to_group (RubricaCard* self, gchar* group);

void rubrica_card_add_group       (RubricaCard* self, gchar* group);
void rubrica_card_add_address     (RubricaCard* self, RubricaAddress* address);
void rubrica_card_add_net_address (RubricaCard* self, RubricaNetAddress* net);
void rubrica_card_add_telephone   (RubricaCard* self, RubricaTelephone* tel);


/*
  TODO
  rubrica_delete/modify_group/address/net_address_telephone
*/

gpointer rubrica_card_get_group            (RubricaCard* self);
gpointer rubrica_card_get_next_group       (RubricaCard* self);
gpointer rubrica_card_get_prev_group       (RubricaCard* self);
void     rubrica_card_reset_group          (RubricaCard* self);

gpointer rubrica_card_get_address          (RubricaCard* self);
gpointer rubrica_card_get_next_address     (RubricaCard* self);
gpointer rubrica_card_get_prev_address     (RubricaCard* self);
void     rubrica_card_reset_address        (RubricaCard* self);

gpointer rubrica_card_get_net_address      (RubricaCard* self);
gpointer rubrica_card_get_next_net_address (RubricaCard* self);
gpointer rubrica_card_get_prev_net_address (RubricaCard* self);
void     rubrica_card_reset_net_address    (RubricaCard* self);

gpointer rubrica_card_get_telephone        (RubricaCard* self);
gpointer rubrica_card_get_next_telephone   (RubricaCard* self);
gpointer rubrica_card_get_prev_telephone   (RubricaCard* self);
void     rubrica_card_reset_telephone      (RubricaCard* self);


/* test metodi
 */
gint rubrica_card_info_type (RubricaCard* self);
void rubrica_card_print     (RubricaCard* self);


#endif
