/*
*  RAL -- Rubrica Addressbook Library
*  file: company.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "card.h"
#include "company.h"
#include "date.h"
#include "groups.h"
#include "address.h"
#include "contact.h"
#include "net.h"
#include "notes.h"
#include "telephone.h"
#include "work.h"
#include "utils.h"
#include "save.h"
#include "ref.h"

struct _RubricaCompanyCardPrivate {
  GList* refs;
  GList* refs_iter;

  gboolean dispose_has_run;
};



static void rubrica_company_card_class_init (RubricaCompanyCardClass* klass);
static void rubrica_company_card_init       (RubricaCompanyCard* obj);

static void rubrica_company_card_dispose    (RubricaCompanyCard* obj);
static void rubrica_company_card_finalize   (RubricaCompanyCard* obj);

static gint rubrica_company_info            (RubricaCompanyCard* self);
static void rubrica_company_card_free       (RubricaCompanyCard* self); 
static void rubrica_company_print           (RubricaCompanyCard* card);

RubricaCompanyCard* rubrica_company_card_copy (RubricaCompanyCard* self);


static RubricaCardClass* parent_class = NULL;



GType
rubrica_company_card_get_type()
{
  static GType company_card_type = 0;
  
  if (!company_card_type)
    {
      static const GTypeInfo company_card_info =
	{
	  sizeof(RubricaCompanyCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_company_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaCompanyCard),
	  0,
	  (GInstanceInitFunc) rubrica_company_card_init
	};

      company_card_type = g_type_register_static (RUBRICA_CARD_TYPE,
						  "RubricaCompanyCard",
						  &company_card_info, 0);
    }
  
  return company_card_type;
}


static void
rubrica_company_card_class_init(RubricaCompanyCardClass* klass)
{
  GObjectClass *object_class;
  
  parent_class             = RUBRICA_CARD_CLASS(klass);
  parent_class->free       = rubrica_company_card_free;
  parent_class->copy       = rubrica_company_card_copy;
  parent_class->write_node = rubrica_save_write_company_data;

  parent_class->info_type  = rubrica_company_info;
  parent_class->print_card = rubrica_company_print;
   
  object_class           = G_OBJECT_CLASS (klass);
  object_class->dispose  = (GObjectFinalizeFunc) rubrica_company_card_dispose;
  object_class->finalize = (GObjectFinalizeFunc) rubrica_company_card_finalize;
}


static void
rubrica_company_card_init(RubricaCompanyCard* self)
{
  self->company_name = NULL;
  self->pi           = NULL;
  self->notes        = NULL;

  self->private = g_new(RubricaCompanyCardPrivate, 1);
  self->private->refs            = NULL;
  self->private->refs_iter       = NULL;
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_company_card_dispose (RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
    
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_company_card_finalize (RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  rubrica_utils_free_string(self->company_name);
  rubrica_utils_free_string(self->pi);  
  rubrica_utils_free_string(self->notes);

  for(; self->private->refs_iter; 
      self->private->refs_iter = self->private->refs_iter->next)
    if(self->private->refs_iter->data)
      rubrica_ref_free(RUBRICA_REF(self->private->refs_iter->data));

  g_list_free(self->private->refs);
  g_list_free(self->private->refs_iter);

  self->private->refs = self->private->refs_iter = NULL;
}



/*  private functions
*/
static void
rubrica_company_card_free(RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  g_object_unref(G_OBJECT(self));
}



static gint 
rubrica_company_info(RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), UNKNOW_CARD);

  return COMPANY_CARD;
}


RubricaCompanyCard* 
rubrica_company_card_copy (RubricaCompanyCard* self)
{
  RubricaCompanyCard* new;
  gpointer data;

  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);
 
  new = rubrica_company_card_new();

  rubrica_company_card_set(new,
			   rubrica_company_card_get_company_name(self),
			   rubrica_company_card_get_pi(self),
			   rubrica_company_card_get_notes(self));

  data = rubrica_company_card_get_ref(RUBRICA_COMPANY_CARD(self));
  for (; data; data = rubrica_company_card_get_next_ref(self))
    {
      RubricaRef* ref;

      ref = rubrica_ref_copy(RUBRICA_REF(data));
      if (ref)
	rubrica_company_card_add_ref(new, ref);
    }

  return new;  
}


/*  Public functions
*/
RubricaCompanyCard*
rubrica_company_card_new(void)
{
  RubricaCompanyCard* self;
  gulong id = 0L;
  gchar* cc_id = NULL;  /* Company Card id */

  self = g_object_new(rubrica_company_card_get_type(), NULL);

  id = RUBRICA_CARD_CLASS(parent_class)->generate_id(RUBRICA_CARD(self), 
						     "company");
  
  if (id == 0L)
    return NULL;

  cc_id = g_strdup_printf("cc%ld", id); 
  rubrica_card_set_id(RUBRICA_CARD(self), cc_id); 
  
  g_free(cc_id);
  cc_id = NULL;
  
  return self;
}


void
rubrica_company_card_set (RubricaCompanyCard* self, gchar* name,
			  gchar* pi, gchar* notes)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
  
  rubrica_company_card_set_company_name (self, name);
  rubrica_company_card_set_pi (self, pi);
  rubrica_company_card_set_notes(self, notes);
}

void
rubrica_company_card_set_company_name (RubricaCompanyCard* self, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  rubrica_utils_free_string(self->company_name);
  self->company_name = g_strdup(name);
}


void
rubrica_company_card_set_pi (RubricaCompanyCard* self, gchar* pi)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  rubrica_utils_free_string(self->pi);
  self->pi = g_strdup(pi);
}


void 
rubrica_company_card_set_notes (RubricaCompanyCard* self, gchar* notes)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
  
  rubrica_utils_free_string(self->notes);
  self->notes = g_strdup(notes);
}



void   
rubrica_company_card_get (RubricaCompanyCard* self, 
			  gchar** name, gchar** pi, gchar** notes)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
  
  if (name)
    *name = rubrica_company_card_get_company_name(self);
  if(pi)
    *pi = rubrica_company_card_get_pi(self);
  if(notes)
    *notes = rubrica_company_card_get_notes(self);
}


gchar* 
rubrica_company_card_get_company_name (RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);
  
  return self->company_name;
}


gchar* 
rubrica_company_card_get_pi (RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);
  
  return self->pi;
}


gchar* 
rubrica_company_card_get_notes (RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);

  return self->notes;
}



void 
rubrica_company_card_add_ref (RubricaCompanyCard* self, RubricaRef* ref)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));
  g_return_if_fail(IS_RUBRICA_REF(ref));

  self->private->refs = g_list_append(self->private->refs, ref);
  self->private->refs_iter = self->private->refs;
}


void 
rubrica_company_card_del_ref (RubricaCompanyCard* self, RubricaRef* ref)
{
  // todo
}

gpointer 
rubrica_company_card_get_ref (RubricaCompanyCard* self)
{
  RubricaRef* ret = NULL;

  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);

  if (self->private->refs_iter)
    ret = self->private->refs_iter->data;
  if (IS_RUBRICA_REF(ret))
    return ret;

  return NULL;
}


gpointer 
rubrica_company_card_get_next_ref (RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);

  self->private->refs_iter = g_list_next(self->private->refs_iter);
  if (self->private->refs_iter)
    return self->private->refs_iter->data;
  else
    self->private->refs_iter = self->private->refs;

  return NULL;
}


gpointer 
rubrica_company_card_get_prev_ref (RubricaCompanyCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_COMPANY_CARD(self), NULL);

  self->private->refs_iter = g_list_previous(self->private->refs_iter);
  if (self->private->refs_iter)
    return self->private->refs_iter->data;
  else
    self->private->refs_iter = self->private->refs;

  return NULL;
}


void     
rubrica_company_card_reset_ref (RubricaCompanyCard* self)
{
  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(self));

  self->private->refs_iter = self->private->refs;
}






/* test */
static void 
rubrica_company_print(RubricaCompanyCard* card)
{
  gpointer data;

  printf("\n\n*** Printing Company Card data ***");
  printf("\ncard id: %s", rubrica_card_get_id(RUBRICA_CARD(card)));
  printf("\nname: %s\t rank: %d\tdeletable: %d\tdeleted: %d",
	 rubrica_card_get_card_name(RUBRICA_CARD(card)),
	 rubrica_card_get_rank(RUBRICA_CARD(card)),
	 rubrica_card_is_deletable(RUBRICA_CARD(card)),
	 rubrica_card_is_deleted(RUBRICA_CARD(card)));  
  printf("\ncard creation time: %ld\tmodification time: %ld", 
	 rubrica_card_get_creation_time(RUBRICA_CARD(card)),
	 rubrica_card_get_modification_time(RUBRICA_CARD(card)));

  printf("\n\n****** Groups");
  data = rubrica_card_get_group(RUBRICA_CARD(card));
  for(; data; data = rubrica_card_get_next_group(RUBRICA_CARD(card)))    
    printf("\ncard belongs to group: %s", (gchar *) data);
  
  printf("\n\n****** Addresses");
  data = rubrica_card_get_address(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_address(RUBRICA_CARD(card)))
    {
      printf("\naddress type:     %s", 
	     rubrica_address_get_address_type(RUBRICA_ADDRESS(data))); 
      printf("\nstreet: %s\tnumber: %s", 
	     rubrica_address_get_street(RUBRICA_ADDRESS(data)), 
	     rubrica_address_get_number(RUBRICA_ADDRESS(data))); 
      printf("\ncity: %s\tzip: %s", 
	     rubrica_address_get_city(RUBRICA_ADDRESS(data)), 
	     rubrica_address_get_zip(RUBRICA_ADDRESS(data))); 
      printf("\nprovince: %s\tstate: %s\tcountry: %s", 
	     rubrica_address_get_province(RUBRICA_ADDRESS(data)),
	     rubrica_address_get_state(RUBRICA_ADDRESS(data)),
	     rubrica_address_get_country(RUBRICA_ADDRESS(data)));      
    }

  printf("\n\n****** Company Data");
  printf("\ncompany name: %s\tpartita iva: %s\tnotes: %s", 
	 rubrica_company_card_get_company_name(card),
	 rubrica_company_card_get_pi(card),
	 rubrica_company_card_get_notes(card));
  data = rubrica_company_card_get_ref(card);
  for (;data; data = rubrica_company_card_get_next_ref(card))
    printf("\nreference -- id: %s\tinfo: %s",
	   rubrica_ref_get_id(RUBRICA_REF(data)),
	   rubrica_ref_get_info(RUBRICA_REF(data)));
   
  printf("\n\n****** Net Addresses");
  data = rubrica_card_get_net_address (RUBRICA_CARD(card)); 
  for (; data; data = rubrica_card_get_next_net_address (RUBRICA_CARD(card)))
    {
      gchar* type;
      
      type = rubrica_net_address_get_url_type(RUBRICA_NET_ADDRESS(data));
      
      printf("\nurl: %s (url type: %s, url type code %d)", 
	     rubrica_net_address_get_url(RUBRICA_NET_ADDRESS(data)), 
	     type, rubrica_net_address_get_decoded_type(type)); 
    }

  printf("\n\n****** Telephone");
  data = rubrica_card_get_telephone(RUBRICA_CARD(card)); 
  for (; data; data = rubrica_card_get_next_telephone(RUBRICA_CARD(card)))
    printf("\ntelephone number (%s): %s",
	   rubrica_telephone_get_tel_type(RUBRICA_TELEPHONE(data)),
	   rubrica_telephone_get_number(RUBRICA_TELEPHONE(data))); 

  printf("\n\n");
}
