/*
*  RAL -- Rubrica Addressbook Library
*  file: contact.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "contact.h"
#include "date.h"
#include "utils.h"


struct _RubricaContactPrivate {
  gboolean dispose_has_run;
};



static void rubrica_contact_class_init (RubricaContactClass* klass);
static void rubrica_contact_init       (RubricaContact* obj);

static void rubrica_contact_dispose    (RubricaContact* obj);
static void rubrica_contact_finalize   (RubricaContact* obj);


GType
rubrica_contact_get_type()
{
  static GType contact_type = 0;
  
  if (!contact_type)
    {
      static const GTypeInfo contact_info =
	{
	  sizeof(RubricaContactClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_contact_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaContact),
	  0,
	  (GInstanceInitFunc) rubrica_contact_init
	};

      contact_type = g_type_register_static (G_TYPE_OBJECT, "RubricaContact",
					     &contact_info, 0);
    }
  
  return contact_type;
}


static void
rubrica_contact_class_init(RubricaContactClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_contact_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_contact_finalize;
}


static void
rubrica_contact_init(RubricaContact* self)
{
  self->first    = NULL;
  self->middle   = NULL;
  self->last     = NULL;
  self->nick     = NULL;
  self->prof     = NULL;
  self->prefix   = NULL;
  self->title    = NULL;
  self->photo    = NULL;
  self->birthday = rubrica_date_new();

  self->private = g_new(RubricaContactPrivate, 1);
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_contact_dispose (RubricaContact* self)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));
  
  if (self->private->dispose_has_run)
    return;

  rubrica_date_free(self->birthday);

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_contact_finalize (RubricaContact* self)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->first); 
  rubrica_utils_free_string(self->middle);
  rubrica_utils_free_string(self->last);
  rubrica_utils_free_string(self->nick);
  rubrica_utils_free_string(self->prof);
  rubrica_utils_free_string(self->prefix);
  rubrica_utils_free_string(self->title);
  rubrica_utils_free_string(self->photo);

  g_free(self->private);
  self->private = NULL;
}


RubricaContact*
rubrica_contact_new(void)
{
  RubricaContact* self;

  self = g_object_new(rubrica_contact_get_type(), NULL);

  return self;
}


void
rubrica_contact_free(RubricaContact* self)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  g_object_unref(self);   
}



void 
rubrica_contact_set (RubricaContact* self, gchar* first, gchar* middle, 
		     gchar* last, gchar* nick, gchar* prof, gchar* prefix, 
		     gchar* title, gchar* photo, time_t birth)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_contact_set_first_name(self, first);
  rubrica_contact_set_middle_name(self, middle);
  rubrica_contact_set_last_name(self, last);
  rubrica_contact_set_nick_name(self, nick);
  rubrica_contact_set_profession(self, prof);
  rubrica_contact_set_prefix(self, prefix);
  rubrica_contact_set_title(self, title);
  rubrica_contact_set_photo(self, photo);
  rubrica_contact_set_birthday(self, birth);
}


void 
rubrica_contact_set_first_name (RubricaContact* self, gchar* first)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->first);
  self->first = g_strdup(first);
}


void 
rubrica_contact_set_middle_name (RubricaContact* self, gchar* middle)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->middle);
  self->middle = g_strdup(middle);
}


void
rubrica_contact_set_last_name (RubricaContact* self, gchar* last)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->last);
  self->last = g_strdup(last);
}


void 
rubrica_contact_set_nick_name (RubricaContact* self, gchar* nick)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->nick);
  self->nick = g_strdup(nick);
}


void 
rubrica_contact_set_profession (RubricaContact* self, gchar* prof)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->prof);
  self->prof = g_strdup(prof);
}


void 
rubrica_contact_set_prefix (RubricaContact* self, gchar* prefix)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));
  
  rubrica_utils_free_string(self->prefix);
  self->prefix = g_strdup(prefix);
}


void 
rubrica_contact_set_title (RubricaContact* self, gchar* title)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->title);
  self->title = g_strdup(title);
}


void 
rubrica_contact_set_photo (RubricaContact* self, gchar* photo)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  rubrica_utils_free_string(self->photo);
  self->photo = g_strdup(photo);
}


void 
rubrica_contact_set_birthday (RubricaContact* self, time_t birth)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  if (birth != -1)
    {
      rubrica_date_know_date(self->birthday, TRUE);
      rubrica_date_set_date(self->birthday, birth);
    }
  else
    rubrica_date_know_date(self->birthday, FALSE);
}



void 
rubrica_contact_get(RubricaContact* self, 
		    gchar** first, gchar** middle, gchar** last, gchar** nick, 
		    gchar** prof,  gchar** prefix, gchar** title, 
		    gchar** photo, gchar** day, gchar** month, gchar** year)
{
  g_return_if_fail(IS_RUBRICA_CONTACT(self));

  if (first)
    *first = rubrica_contact_get_first_name(self);
  if (middle)
    *middle = rubrica_contact_get_middle_name(self);
  if (last)
    *last = rubrica_contact_get_last_name(self);
  if (nick)
    *nick = rubrica_contact_get_nick_name(self);
  if (prof)
    *prof = rubrica_contact_get_profession(self);
  if (prefix)
    *prefix = rubrica_contact_get_prefix(self);
  if (title)
    *title = rubrica_contact_get_title(self);
  if (photo)
    *photo = rubrica_contact_get_photo(self);
  if (day)
    *day = rubrica_contact_get_birth_day(self);
  if (month)
    *month = rubrica_contact_get_birth_month(self);
  if (year)
    *year = rubrica_contact_get_birth_year(self);
}


gchar* 
rubrica_contact_get_first_name (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->first;
}

gchar* 
rubrica_contact_get_middle_name (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->middle;
}


gchar* 
rubrica_contact_get_last_name (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->last;
}


gchar* 
rubrica_contact_get_nick_name (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->nick;
}


gchar* 
rubrica_contact_get_profession (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->prof;
}


gchar* 
rubrica_contact_get_prefix (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->prefix;
}


gchar* 
rubrica_contact_get_title (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->title;
}


gchar* 
rubrica_contact_get_photo (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return self->photo;
}


gchar* 
rubrica_contact_get_birth_day (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return rubrica_date_get_day(self->birthday);
}


gchar* 
rubrica_contact_get_birth_month (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return rubrica_date_get_month(self->birthday);
}


gchar* 
rubrica_contact_get_birth_year (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);
  
  return rubrica_date_get_year(self->birthday);
}


time_t
rubrica_contact_get_birthdate (RubricaContact* self)
{
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), (time_t) -1);

  return rubrica_date_get_date(self->birthday); 
}


RubricaContact* 
rubrica_contact_copy (RubricaContact* self)
{
  RubricaContact* new;
  
  g_return_val_if_fail(IS_RUBRICA_CONTACT(self), NULL);

  new = rubrica_contact_new();

  rubrica_contact_set(new, 
		      rubrica_contact_get_first_name(self),
		      rubrica_contact_get_middle_name(self),
		      rubrica_contact_get_last_name(self),
		      rubrica_contact_get_nick_name(self),
		      rubrica_contact_get_profession(self),
		      rubrica_contact_get_prefix(self),
		      rubrica_contact_get_title(self),
		      rubrica_contact_get_photo(self),
		      rubrica_contact_get_birthdate(self));

  return new;
}
