/*
*  RAL -- Rubrica Addressbook Library
*  file: csv.c
*
*  the vcard import engine v0.5 
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <glib.h>
#include <glib-object.h>

#include "csv.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "work.h"
#include "notes.h"
#include "address.h"
#include "net.h"
#include "telephone.h"
#include "ref.h"


#define COMMA_DELIM ','
#define TAB_DELIM   '\t'
#define QUOTES      '"'

typedef enum {
  CSV_FIRST_NAME = 1,           // 1
  CSV_LAST_NAME,                // 2
  CSV_DISPLAY_NAME,             // 3
  CSV_NICKNAME,                 // 4
  CSV_EMAIL1,                   // 5
  CSV_EMAIL2,                   // 6
  CSV_TEL_WORK,                 // 7
  CSV_TEL_HOME,                 // 8
  CSV_TEL_FAX,                  // 9
  CSV_TEL_PAGER,                // 10
  CSV_TEL_MOBILE,               // 11
  CSV_HOME_STREET,              // 12
  CSV_HOME_STREET_MORE,         // 13
  CSV_HOME_CITY,                // 14
  CSV_HOME_STATE_PROVINCE,      // 15
  CSV_HOME_ZIP,                 // 16
  CSV_HOME_COUNTRY,             // 17
  CSV_WORK_STREET,              // 18
  CSV_WORK_STREET_MORE,         // 19
  CSV_WORK_CITY,                // 20
  CSV_WORK_STATE_PROVINCE,      // 21
  CSV_WORK_ZIP,                 // 22
  CSV_WORK_COUNTRY,             // 23
  CSV_TITLE,                    // 24
  CSV_DEPARTMENT,               // 25
  CSV_ORGANIZATION,             // 26
  CSV_WORK_WEB,                 // 27
  CSV_WEB,                      // 28
  CSV_UNDEF1,                   // 29
  CSV_UNDEF2,                   // 30
  CSV_UNDEF3,                   // 31
  CSV_CUSTOM1,                  // 32
  CSV_CUSTOM2,                  // 33
  CSV_CUSTOM3,                  // 34
  CSV_CUTOM4,                   // 35
  CSV_NOTE,                     // 36
  CSV_MORE,                     // 37
} RubricaCsvField;


typedef enum {
  RUBRICA_CSV_START,
  RUBRICA_CSV_QUOTES,
  RUBRICA_CSV_DQUOTES,
  RUBRICA_CSV_CLOSE_QUOTES,
  RUBRICA_CSV_CLOSE_DQUOTES,  
  RUBRICA_CSV_BUILD,
  RUBRICA_CSV_BUILD_QUOTES,
  RUBRICA_CSV_BUILD_DQUOTES,
  RUBRICA_CSV_STORE                    // not used state
} RubricaCsvState;


/*    signals enumeration 
 */
enum {
  CSV_FILE_OPENED,              // csv file was/wasn't opened
  CSV_CARD_DECODED,             // csv item (a file's row) was decoded
  CSV_ADDRESSBOOK_DECODED,      // csv file was decoded
  LAST_SIGNAL
};

static guint rubrica_csv_signals[LAST_SIGNAL] = {0};

static GObject* parent_class = NULL;


struct _RubricaCsvPrivate{
  RubricaCsvState state;    /* automoton's state */

  RubricaCsvField field;

  glong file_pos;
  GString* buffer;          /* data currently read  */

  gboolean dispose_has_run;
};


static gboolean new_card = TRUE;
static RubricaPersonalCard* card         = NULL;
static RubricaContact*      contact      = NULL;
static RubricaAddress*      home_address = NULL;
static RubricaAddress*      work_address = NULL;
static RubricaWork*         work         = NULL;
static RubricaNotes*        notes        = NULL;
static RubricaNetAddress*   net          = NULL;
static RubricaTelephone*    phone        = NULL;


static void   rubrica_csv_init        (RubricaCsv* obj);
static void   rubrica_csv_class_init  (RubricaCsvClass* klass);

static void   rubrica_csv_dispose     (GObject* obj);
static void   rubrica_csv_finalize    (GObject* obj);



/* private functions
*/
static gboolean        rubrica_csv_is_separator (gunichar ch);
static gboolean        rubrica_csv_is_quote     (gunichar ch);
static RubricaCsvState rubrica_csv_get_state    (RubricaCsv* self);

static void rubrica_csv_store_decoded_data  (RubricaCsv* self, 
					     RubricaPersonalCard* card);
static void rubrica_csv_append_unichar      (RubricaCsv* self, gunichar ch);
static void rubrica_csv_clean_buffer        (RubricaCsv* self);
static void rubrica_csv_set_next_state      (RubricaCsv* self, 
					     RubricaCsvState state);
static void rubrica_csv_build_data          (RubricaCsv* self, gunichar ch);
static void rubrica_csv_store_file_position (RubricaCsv* self);
gint        rubrica_csv_open_file           (RubricaCsv* self, char* filename);



GType
rubrica_csv_get_type()
{
  static GType rubrica_csv_type = 0;
  
  if (!rubrica_csv_type)
    {
      static const GTypeInfo rubrica_csv_info =
	{
	  sizeof(RubricaCsvClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_csv_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaCsv),
	  0,
	  (GInstanceInitFunc) rubrica_csv_init
	};

      rubrica_csv_type = g_type_register_static (G_TYPE_OBJECT, "RubricaCsv",
						 &rubrica_csv_info, 0);
    }
  
  return rubrica_csv_type;
}


static void
rubrica_csv_class_init(RubricaCsvClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_csv_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_csv_finalize;

  parent_class = g_type_class_peek_parent (klass);

  rubrica_csv_signals[CSV_FILE_OPENED] = 
    g_signal_new("file_opened", 
		 RUBRICA_CSV_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaCsvClass, csv_file_opened),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,         /* return type */
		 1,                   /* params      */
		 G_TYPE_INT);         /* params type */

  rubrica_csv_signals[CSV_CARD_DECODED] = 
    g_signal_new("card_decoded", 
		 RUBRICA_CSV_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaCsvClass, csv_card_decoded),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
  
  rubrica_csv_signals[CSV_ADDRESSBOOK_DECODED] =     
    g_signal_new("addressbook_decoded", 
		 RUBRICA_CSV_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RubricaCsvClass, csv_addressbook_decoded),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);
}


static void
rubrica_csv_init(RubricaCsv* self)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  self->fp  = NULL;
  
  self->private = g_new(RubricaCsvPrivate, 1);
  if (!self->private)
    g_error("\nOut of memory");

  self->private->field    = CSV_FIRST_NAME;
  self->private->state    = RUBRICA_CSV_START;
  self->private->file_pos = 0L;  
  self->private->buffer   = g_string_new(NULL);
  
  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_csv_dispose (GObject* obj)
{
  RubricaCsv* self;
 
  self = RUBRICA_CSV(obj);
  g_return_if_fail(IS_RUBRICA_CSV(self));

  if (self->private->dispose_has_run)
    return;

  g_string_free(self->private->buffer, TRUE);
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_csv_finalize (GObject* obj)
{
  RubricaCsv* self;

  self = RUBRICA_CSV(obj);
  g_return_if_fail(IS_RUBRICA_CSV(self));
  
  g_free(self->private);
  if (self->fp)
    fclose(self->fp);
}


static gboolean 
rubrica_csv_is_separator(gunichar ch)
{
  if ((ch == COMMA_DELIM) ||   /* comma separator */
      (ch == TAB_DELIM))       /* tab   separator */
    return TRUE;
  
  return FALSE;
}


static gboolean 
rubrica_csv_is_quote(gunichar ch)
{
  if (ch == QUOTES) 
    return TRUE;
  
  return FALSE;
}


static RubricaCsvState
rubrica_csv_get_state(RubricaCsv* self)
{
  g_return_val_if_fail(IS_RUBRICA_CSV(self), RUBRICA_CSV_START);

  return (RubricaCsvState) self->private->state;
}


static void 
rubrica_csv_store_decoded_data(RubricaCsv* self, RubricaPersonalCard* card)
{
  GString* string;
  gchar* more = NULL;
  gchar* tmp = NULL;

  g_return_if_fail(IS_RUBRICA_CSV(self));

  string = self->private->buffer;
  if (string->str)
    tmp = g_strdup(string->str);
  
  if (tmp && (g_ascii_strcasecmp(tmp, "") != 0))
    {  
      switch (self->private->field)
	{
	case CSV_FIRST_NAME:
	  rubrica_contact_set_first_name(contact, tmp);
	  break;
	  
	case CSV_LAST_NAME:
	  rubrica_contact_set_last_name(contact, tmp);
	  break;
	  
	case CSV_DISPLAY_NAME:
	  rubrica_card_set_card_name(RUBRICA_CARD(card), tmp);
	  break;
	  
	case CSV_NICKNAME:
	  rubrica_contact_set_nick_name(contact, tmp);
	  break;
	  
	case CSV_EMAIL1:
	case CSV_EMAIL2:
	  net = rubrica_net_address_new();
	  rubrica_net_address_set(net, tmp, RUBRICA_NET_ADDRESS_EMAIL);
	  rubrica_card_add_net_address(RUBRICA_CARD(card), net);
	  break;
	  
	case CSV_TEL_WORK:
	  phone = rubrica_telephone_new();
	  rubrica_telephone_set(phone, tmp, "work");
	  rubrica_card_add_telephone(RUBRICA_CARD(card), phone);
	  break;
	  
	case CSV_TEL_HOME:
	  phone = rubrica_telephone_new();
	  rubrica_telephone_set(phone, tmp, "home");
	  rubrica_card_add_telephone(RUBRICA_CARD(card), phone);	  
	  break;
	  
	case CSV_TEL_FAX:
	  phone = rubrica_telephone_new();
	  rubrica_telephone_set(phone, tmp, "fax");
	  rubrica_card_add_telephone(RUBRICA_CARD(card), phone);
	  break;
	  
	case CSV_TEL_PAGER:
	  phone = rubrica_telephone_new();
	  rubrica_telephone_set(phone, tmp, "other");
	  rubrica_card_add_telephone(RUBRICA_CARD(card), phone);
	  break;
	  
	case CSV_TEL_MOBILE:
	  phone = rubrica_telephone_new();
	  rubrica_telephone_set(phone, tmp, "cellphone");
	  rubrica_card_add_telephone(RUBRICA_CARD(card), phone);
	  break;
	  
	case CSV_HOME_STREET:
	  home_address = rubrica_address_new();
	  rubrica_address_set_address_type(home_address, RUBRICA_ADDRESS_HOME);
	  rubrica_address_set_street(home_address, tmp);
	  break;
	  
	case CSV_HOME_STREET_MORE:
	  if (!home_address)
	    {
	      home_address = rubrica_address_new();
	      rubrica_address_set_address_type(home_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  more = g_strdup_printf("%s %s", 
				 rubrica_address_get_street(home_address),
				 tmp);
	  
	  rubrica_address_set_street(home_address, more);
	  g_free(more);	  
	  break;
	  
	case CSV_HOME_CITY:
	  if (!home_address)
	    {
	      home_address = rubrica_address_new();
	      rubrica_address_set_address_type(home_address,
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_city(home_address, tmp);
	  break;
	  
	case CSV_HOME_STATE_PROVINCE:
	  if (!home_address)
	    {
	      home_address = rubrica_address_new();
	      rubrica_address_set_address_type(home_address,
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_province(home_address, tmp);
	  break;
	  
	case CSV_HOME_ZIP:
	  if (!home_address)
	    {
	      home_address = rubrica_address_new();
	      rubrica_address_set_address_type(home_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_zip(home_address, tmp);
	  break;
	  
	case CSV_HOME_COUNTRY:
	  if (!home_address)
	    {
	      home_address = rubrica_address_new();
	      rubrica_address_set_address_type(home_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_country(home_address, tmp);
	  rubrica_card_add_address(RUBRICA_CARD(card), home_address);
	  break;
	  
	case CSV_WORK_STREET:
	  work_address = rubrica_address_new();
	  rubrica_address_set_address_type(work_address, RUBRICA_ADDRESS_WORK);
	  rubrica_address_set_street(work_address, tmp);
	  break;
	  
	case CSV_WORK_STREET_MORE:
	  if (!work_address)
	    {
	      work_address = rubrica_address_new();
	      rubrica_address_set_address_type(work_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  more = g_strdup_printf("%s %s", 
				 rubrica_address_get_street(work_address),
				 tmp);
	  
	  rubrica_address_set_street(work_address, more);
	  g_free(more);	  	  
	  break;
	  
	case CSV_WORK_CITY:
	  if (!work_address)
	    {
	      work_address = rubrica_address_new();
	      rubrica_address_set_address_type(work_address,
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_city(work_address, tmp);	  
	  break;
	  
	case CSV_WORK_STATE_PROVINCE:
	  if (!work_address)
	    {
	      work_address = rubrica_address_new();
	      rubrica_address_set_address_type(work_address,
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_province(work_address, tmp);
	  break;
	  
	case CSV_WORK_ZIP:
	  if (!work_address)
	    {
	      work_address = rubrica_address_new();
	      rubrica_address_set_address_type(work_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_zip(work_address, tmp);	  
	  break;
	  
	case CSV_WORK_COUNTRY:
	  if (!work_address)
	    {
	      work_address = rubrica_address_new();
	      rubrica_address_set_address_type(work_address, 
					       RUBRICA_ADDRESS_HOME);
	    }
	  rubrica_address_set_country(work_address, tmp);
	  rubrica_card_add_address(RUBRICA_CARD(card), work_address);
	  break;
	  
	case CSV_TITLE:
	  rubrica_work_set_assigment(work, tmp); 
	  break;
	  
	case CSV_DEPARTMENT:
	  rubrica_work_set_department(work, tmp);
	  break;
	  
	case CSV_ORGANIZATION:
	  rubrica_work_set_organization(work, tmp);
	  break;
	  
	case CSV_WORK_WEB:
	case CSV_WEB:
	  net = rubrica_net_address_new();
	  rubrica_net_address_set(net, tmp, RUBRICA_NET_ADDRESS_WEB);
	  rubrica_card_add_net_address(RUBRICA_CARD(card), net);
	  break;
	  
	case CSV_UNDEF1:
	  break;
	  
	case CSV_UNDEF2:
	  break;
	  
	case CSV_UNDEF3:
	  break;
	  
	case CSV_CUSTOM1:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	case CSV_CUSTOM2:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	case CSV_CUSTOM3:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	case CSV_CUTOM4:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	case CSV_NOTE:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	case CSV_MORE:
	  rubrica_notes_append_other_notes(notes, tmp);
	  break;
	  
	default:
	  g_free(tmp);
	  break;
	}
    }
  
  if (self->private->field != CSV_MORE)
    self->private->field++;
  else
    self->private->field = CSV_FIRST_NAME;
}


static void
rubrica_csv_append_unichar(RubricaCsv* self, gunichar ch)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  g_string_append_unichar(self->private->buffer, ch);
}


static void 
rubrica_csv_set_next_state(RubricaCsv* self, RubricaCsvState state)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  self->private->state = state;
}



static void 
rubrica_csv_clean_buffer (RubricaCsv* self)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));
  
  if (self->private->buffer)
    g_string_truncate (self->private->buffer, 0);
}


static void 
rubrica_csv_build_data(RubricaCsv* self, gunichar ch)
{
  RubricaCsvState current_state;

  g_return_if_fail(IS_RUBRICA_CSV(self));

  current_state = rubrica_csv_get_state(self);
  switch (current_state)
    {
    case RUBRICA_CSV_START:
      if (new_card)
	{
	  card    = rubrica_personal_card_new(); 
	  contact = rubrica_contact_new();
	  work    = rubrica_work_new();
	  notes   = rubrica_notes_new();
	  
	  rubrica_personal_card_set_contact(card, contact);
	  rubrica_personal_card_set_work(card, work);
	  rubrica_personal_card_set_notes(card, notes);
	  
	  new_card = FALSE;
	}
      
      rubrica_csv_clean_buffer(self);      
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_QUOTES);	
      else
	{
	  if (rubrica_csv_is_separator(ch))
	    {
	      rubrica_csv_store_decoded_data(self, card);
	      rubrica_csv_set_next_state(self, RUBRICA_CSV_START);
	    }
	  else
	    {
	      rubrica_csv_append_unichar(self, ch);
	      rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD);
	    }
	}
      break;
      
    case RUBRICA_CSV_QUOTES:
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_DQUOTES);	
      else
	{
	  rubrica_csv_append_unichar(self, ch);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD_QUOTES);
	}
      break;
      
    case RUBRICA_CSV_DQUOTES:
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_DQUOTES);
      else
	{
	  if (rubrica_csv_is_separator(ch))
	    {
	      rubrica_csv_store_decoded_data(self, card);
	      rubrica_csv_set_next_state(self, RUBRICA_CSV_START);
	    }
	  else
	    {
	      rubrica_csv_append_unichar(self, ch);
	      rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD_DQUOTES);
	    }
	}     
      break;
      
    case RUBRICA_CSV_BUILD:
      if (rubrica_csv_is_separator(ch))
	{
	  rubrica_csv_store_decoded_data(self, card);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_START);
	}
      else
	{
	  rubrica_csv_append_unichar(self, ch);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD);
	}               
      break;	  
      
    case RUBRICA_CSV_BUILD_QUOTES:
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_QUOTES);
      else
	{
	  rubrica_csv_append_unichar(self, ch);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD_QUOTES);
	}       
      break;	  
      
    case RUBRICA_CSV_BUILD_DQUOTES:
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_DQUOTES);
      else
	{
	  rubrica_csv_append_unichar(self, ch);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_BUILD_DQUOTES);
	} 
      break;	 

    case RUBRICA_CSV_CLOSE_QUOTES:
      if (!rubrica_csv_is_separator(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_QUOTES);
      else
	{
	  rubrica_csv_store_decoded_data(self, card);
	  rubrica_csv_set_next_state(self, RUBRICA_CSV_START);
	  /* */
	}
      break;
      
    case RUBRICA_CSV_CLOSE_DQUOTES:
      if (rubrica_csv_is_quote(ch))
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_QUOTES);	      
      else
	rubrica_csv_set_next_state(self, RUBRICA_CSV_CLOSE_DQUOTES);
      break;	  
      
    default:
      break;
    }
}


void 
rubrica_csv_store_file_position(RubricaCsv *self)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  self->private->file_pos = ftell(self->fp);
}


gint          
rubrica_csv_open_file (RubricaCsv* self, gchar* filename)
{
  gunichar ch;

  g_return_val_if_fail(IS_RUBRICA_CSV(self), CSV_INVALID);
  g_return_val_if_fail(filename != NULL, FILENAME_INVALID);

  self->fp = fopen(filename, "r");
  if (!self->fp)
    {
      g_signal_emit_by_name(self, "file_opened", FILE_OPEN_FAIL, G_TYPE_INT);

      return FILE_OPEN_FAIL;
    }
  else
    g_signal_emit_by_name(self, "file_opened", FILE_OPEN_SUCCESS, G_TYPE_INT);
  
  do 
    {
      ch = fgetc(self->fp);
      
      /* read characters from file until \n or \n\r or EOF is reached 
       */
      if ((ch != feof(self->fp)))
	{
	  if ((ch != '\n') && (ch != '\r'))
	    rubrica_csv_build_data(self, ch);
	  else 
	    {
	      rubrica_csv_store_file_position(self);
	      g_signal_emit_by_name(self, "card_decoded", 
				    card, G_TYPE_POINTER);
	      new_card = TRUE;
	      
	    }
	}
    }
  while(!feof(self->fp));

  g_signal_emit_by_name(self, "addressbook_decoded", NULL, G_TYPE_NONE);
  
  return FILE_IMPORT_SUCCESS;
}



/* Public functions
 */
RubricaCsv* 
rubrica_csv_new(void)
{
  return (RubricaCsv*) g_object_new(rubrica_csv_get_type(), NULL);
}


void 
rubrica_csv_free(RubricaCsv *self)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  g_object_unref(self);
}

void        
rubrica_csv_read_previous_record (RubricaCsv *self)
{
  g_return_if_fail(IS_RUBRICA_CSV(self));

  fseek(self->fp, self->private->file_pos, SEEK_SET);
}





