/*
*  RAL -- Rubrica Addressbook Library
*  file: date.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "date.h"
#include "utils.h"


struct _RubricaDatePrivate {
  gboolean know;      /* is date khnown? */
  time_t date;        /* the date */

  gchar* day;                
  gchar* month;
  gchar* year; 

  gboolean dispose_has_run;
};



static void rubrica_date_class_init (RubricaDateClass* klass);
static void rubrica_date_init       (RubricaDate* obj);

static void rubrica_date_dispose    (RubricaDate* obj);
static void rubrica_date_finalize   (RubricaDate* obj);


GType
rubrica_date_get_type()
{
  static GType rubrica_date_type = 0;
  
  if (!rubrica_date_type)
    {
      static const GTypeInfo rubrica_date_info =
	{
	  sizeof(RubricaDateClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_date_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaDate),
	  0,
	  (GInstanceInitFunc) rubrica_date_init
	};

      rubrica_date_type = g_type_register_static (G_TYPE_OBJECT, "RubricaDate",
						  &rubrica_date_info, 0);
    }
  
  return rubrica_date_type;
}


static void
rubrica_date_class_init(RubricaDateClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_date_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_date_finalize;
}


static void
rubrica_date_init(RubricaDate* date)
{
  date->private = g_new(RubricaDatePrivate, 1);

  date->private->know  = FALSE;
  date->private->date  = (time_t) -1;
  date->private->day   = g_strdup("BadDay");   
  date->private->month = g_strdup("BadMonth");
  date->private->year  = g_strdup("BadYear");

  date->private->dispose_has_run = FALSE;
}


static void 
rubrica_date_dispose (RubricaDate* date)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));
  
#ifdef DEBUG
  g_print("\n\t\trubrica_date_dispose");
#endif
  
  if (date->private->dispose_has_run)
    return;

  date->private->dispose_has_run = TRUE;
}


static void 
rubrica_date_finalize (RubricaDate* date)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));

#ifdef DEBUG
  g_print("\n\t\trubrica_date_finalize");
#endif

  rubrica_utils_free_string(date->private->day);
  rubrica_utils_free_string(date->private->month);
  rubrica_utils_free_string(date->private->year);

  g_free(date->private);
  date->private = NULL;
}


RubricaDate*
rubrica_date_new(void)
{
  RubricaDate* date;

  date = g_object_new(rubrica_date_get_type(), NULL);

  return date;
}


void
rubrica_date_free(RubricaDate* date)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));

#ifdef DEBUG
  g_print("\n\t\t**** Free Date");
#endif

  g_object_unref(date);   
}

void 
rubrica_date_know_date (RubricaDate* date, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_DATE(date));

  date->private->know = know;
}


void 
rubrica_date_set_date (RubricaDate* date, time_t the_date)
{
  struct tm *tm;

  g_return_if_fail(IS_RUBRICA_DATE(date));

  if (!date->private->know)
    return;

  date->private->date = the_date;
  tm = localtime(&the_date);

  if (tm)
    {
      rubrica_utils_free_string(date->private->day);
      rubrica_utils_free_string(date->private->month);
      rubrica_utils_free_string(date->private->year);
      
      date->private->day   = g_strdup_printf("%d", tm->tm_mday);
      date->private->month = g_strdup_printf("%d", tm->tm_mon);
      date->private->year  = g_strdup_printf("%d", tm->tm_year);  
    }
}



gboolean 
rubrica_date_is_date_known (RubricaDate* date)
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), FALSE);

  return date->private->know;
}


time_t
rubrica_date_get_date(RubricaDate* date)
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), (time_t) -1);
  
  return date->private->date;
}

gchar*   
rubrica_date_get_day (RubricaDate* date) 
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), NULL);
 
  return date->private->day;
}


gchar*   
rubrica_date_get_month (RubricaDate* date) 
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), NULL);
 
  return date->private->month;
}


gchar*   
rubrica_date_get_year (RubricaDate* date)
{
  g_return_val_if_fail(IS_RUBRICA_DATE(date), NULL);
 
  return date->private->year;
}


RubricaDate* 
rubrica_date_copy (RubricaDate* date)
{
  RubricaDate* new;

  g_return_val_if_fail(IS_RUBRICA_DATE(date), NULL);

  new = rubrica_date_new();

  rubrica_date_know_date(new, rubrica_date_is_date_known(date));
  rubrica_date_set_date(new, rubrica_date_get_date(date));

  return new;
}
