/*
*  RAL -- Rubrica Addressbook Library
*  file: groups.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "groups.h"
#include "utils.h"


struct _RubricaGroupsPrivate {
  GList* groups;
  GList* iter;

  gboolean dispose_has_run;
};


static void rubrica_groups_class_init (RubricaGroupsClass* klass);
static void rubrica_groups_init       (RubricaGroups* obj);

static void rubrica_groups_dispose    (RubricaGroups* obj);
static void rubrica_groups_finalize   (RubricaGroups* obj);



GType
rubrica_groups_get_type()
{
  static GType groups_type = 0;
  
  if (!groups_type)
    {
      static const GTypeInfo groups_info =
	{
	  sizeof(RubricaGroupsClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_groups_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaGroups),
	  0,
	  (GInstanceInitFunc) rubrica_groups_init
	};

      groups_type = g_type_register_static (G_TYPE_OBJECT, 
					    "RubricaGroups",
					    &groups_info, 0);
    }
  
  return groups_type;
}


static void
rubrica_groups_class_init(RubricaGroupsClass* klass)
{
  GObjectClass *class; 

  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_groups_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_groups_finalize;
}


static void
rubrica_groups_init(RubricaGroups* self)
{
  self->private = g_new(RubricaGroupsPrivate, 1);

  self->private->groups = NULL;
  self->private->iter   = NULL;

  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_groups_dispose (RubricaGroups* self)
{
  GList* list;

  g_return_if_fail(IS_RUBRICA_GROUPS(self));
  
  if (self->private->dispose_has_run)
    return;

  for (list = self->private->groups; list; list = g_list_next(list))
    g_free(list->data);
  
  g_list_free(self->private->groups);

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_groups_finalize (RubricaGroups* self)
{
  g_return_if_fail(IS_RUBRICA_GROUPS(self));
  
  g_free(self->private);
  self->private = NULL;
}



/*    Public
*/
RubricaGroups*
rubrica_groups_new(void)
{
  RubricaGroups* self;

  self = g_object_new(rubrica_groups_get_type(), NULL);

  return self;
}


void
rubrica_groups_free(RubricaGroups* self)
{
  g_return_if_fail(IS_RUBRICA_GROUPS(self));

  g_object_unref(self);   
}



RubricaGroups* 
rubrica_groups_copy (RubricaGroups* self)
{
  RubricaGroups* new;
  GList* list;

  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), NULL);

  new = rubrica_groups_new();
  
  for (list = self->private->groups; list; list = list->next)
    rubrica_groups_add_group(new, g_strdup(list->data));
  
  return new;
}



gboolean 
rubrica_groups_owns_group (RubricaGroups* self, gchar* group)
{
  GList* list;

  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), FALSE);

  if (g_ascii_strcasecmp(group, "all groups") == 0)
    return TRUE;

  for (list = self->private->groups; list; list = g_list_next(list))
    if (g_ascii_strcasecmp((char*) list->data, group) == 0)
      return TRUE;
  
  return FALSE;  
}


gboolean               
rubrica_groups_add_group (RubricaGroups* self, gchar* group)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), FALSE);
  g_return_val_if_fail(group != NULL, FALSE);

  if (rubrica_groups_owns_group(self, group))
    return TRUE;
  
  self->private->groups = g_list_append(self->private->groups, 
					g_strdup(group));

  /* init group iterator */
  if (!self->private->iter)
    self->private->iter = self->private->groups;  

  return TRUE;
}


gboolean     
rubrica_groups_delete_group (RubricaGroups* self, gchar* group)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), FALSE); 

  self->private->iter = self->private->groups;
  for (; self->private->iter; self->private->iter = self->private->iter->next) 
    { 
      if (g_ascii_strcasecmp(self->private->iter->data, group) == 0) 
	{ 
	  self->private->groups = g_list_remove_link(self->private->groups,
						     self->private->iter);
	  
	  g_free(self->private->iter->data);
	  g_list_free_1(self->private->iter);
	  self->private->iter = NULL;	  

	  return TRUE;
	}
    }
  
  return FALSE;
}



gboolean    
rubrica_groups_modify_group (RubricaGroups* self, gchar* old, gchar* new)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), FALSE); 

  self->private->iter = self->private->groups;
  for (; self->private->iter; self->private->iter = self->private->iter->next) 
    { 
      if (g_ascii_strcasecmp(self->private->iter->data, old) == 0) 
	{ 
	  g_free(self->private->iter->data);
	  self->private->iter->data = g_strdup(new);

	  return TRUE;
	}
    }
  
  return FALSE;
}



gchar*
rubrica_groups_get_group (RubricaGroups* self)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), NULL); 

  if (self->private->iter)
    return self->private->iter->data;

  return NULL;    
}


gchar*
rubrica_groups_get_next_group(RubricaGroups* self)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), NULL); 

  self->private->iter = g_list_next(self->private->iter);
  if (self->private->iter)
    return self->private->iter->data;
  else
    self->private->iter = self->private->groups;

  return NULL;
}


gchar*
rubrica_groups_get_prev_group(RubricaGroups* self)
{
  g_return_val_if_fail(IS_RUBRICA_GROUPS(self), NULL); 

  self->private->iter = g_list_previous(self->private->iter);
  if (self->private->iter)
    return self->private->iter->data;
  else
    self->private->iter = self->private->groups;

  return NULL;
}



void rubrica_groups_reset (RubricaGroups* self)
{
  g_return_if_fail(IS_RUBRICA_GROUPS(self)); 

  self->private->iter = self->private->groups;
}
