/*
*  RAL -- Rubrica Addressbook Library
*  file: io.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <time.h>
#include <glib.h>
#include <libxml/tree.h>

#include "io.h"


void     
rubrica_io_set_str (xmlNodePtr node, gchar *label, gchar* string)
{
  xmlChar* xmlstr;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  xmlstr = xmlStrdup(string);
  
  xmlNewProp(node, label, xmlstr);
}


void   
rubrica_io_set_bool (xmlNodePtr node, gchar *label, gboolean bool)
{
  xmlChar* xmlstr;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  xmlstr = xmlStrdup((bool ? "true" : "false"));
  
  xmlNewProp(node, label, xmlstr);
}


void     
rubrica_io_set_num (xmlNodePtr node, gchar *label, gulong val)
{
  xmlChar* xmlstr;
  gchar* str;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  str  = g_strdup_printf("%ld", val);
  xmlstr = xmlStrdup(str);
  
  xmlNewProp(node, label, xmlstr);
  g_free(str);
}



void     
rubrica_io_set_date (xmlNodePtr node, gboolean bool, time_t date)
{
  GDate* gdate;
  GDateDay day;
  GDateMonth month;
  GDateYear year;
  
  g_return_if_fail(node != NULL);
  
  rubrica_io_set_bool(node, "known", bool);

  gdate = g_date_new();
  g_date_set_time(gdate, date);
  day   = g_date_get_day(gdate);
  month = g_date_get_month(gdate);
  year  = g_date_get_year(gdate);
  
  if (!bool || day == G_DATE_BAD_DAY)  
    rubrica_io_set_str(node, "day", "BadDay");
  else
    rubrica_io_set_num(node, "day", day);
  
  if (!bool || month == G_DATE_BAD_MONTH)
    rubrica_io_set_str(node, "month", "BadMonth");
  else
    rubrica_io_set_num(node, "month", month);
  
  if (!bool || year == G_DATE_BAD_YEAR)
    rubrica_io_set_str(node, "year", "BadYear");
  else
    rubrica_io_set_num(node, "year", year);
  
  g_date_free(gdate);
}


gchar* 
rubrica_io_get(xmlNodePtr node, xmlChar* name, RubricaError *err)
{
  xmlNodePtr child;

  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, NULL);

  child = node->children;
  if (xmlIsBlankNode(child))
    child = child->next;   

  while (child)
    {
      gchar* ret;

      if (xmlStrcmp(child->name, name) == 0)
	{
	  ret = xmlNodeGetContent(child);
	  
	  if (xmlStrcmp(ret, "") == 0)
	    {
	      xmlFree(ret);

	      return NULL;
	    }

	  return ret;
	}
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
    }

  *err = UNREADABLE_NODE;
  return NULL;
}
     

gchar* 
rubrica_io_get_content(xmlNodePtr node, RubricaError *err)
{
  xmlChar* tmp = NULL;

  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, NULL);

  tmp = xmlNodeGetContent(node);
  if (!tmp || (xmlStrcmp(tmp, "") == 0))
    {
      *err = UNREADABLE_NODE;
      
      return NULL;
    }
  
  *err = NO_ERROR;
  return (gchar *) tmp;  
}



gboolean
rubrica_io_get_bool(xmlNodePtr node, gchar* key, RubricaError *err)
{
  gchar* tmp = NULL;
  gboolean ret = FALSE;

  *err = UNREADABLE_PROPERTY;
  g_return_val_if_fail(node != NULL, FALSE);

  *err = KEY_NOT_VALID;
  g_return_val_if_fail(key != NULL, FALSE);

  tmp = rubrica_io_get_prop(node, key, err);

  if (!tmp)
    {
      *err = UNREADABLE_PROPERTY;
      return FALSE;
    }
  
  *err = NO_ERROR;
  if(xmlStrcmp(tmp, (xmlChar*) "true") == 0)    
    ret = TRUE;
  else
    ret = FALSE;

  g_free(tmp);
  return ret;
}


glong 
rubrica_io_get_date (xmlNodePtr node, gchar* key, RubricaError *err)
{
  gchar* tmp = NULL;
  glong ret;

  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, 0);

  *err = KEY_NOT_VALID;
  g_return_val_if_fail(key  != NULL, 0);

  tmp = rubrica_io_get_prop(node, key, err);
  if (!tmp)
    {
      *err = INVALID_DATE;
      return 0;
    }

  *err = NO_ERROR;
  ret = atol(tmp);
  g_free(tmp);

  return ret;
}


time_t 
rubrica_io_get_calendar(xmlNodePtr node, gchar** day, gchar** month, 
			gchar** year, RubricaError *err)
{
  GDate *gdate;
  struct tm tm;
  gchar *_day, *_month, *_year;
  
  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, 0);

  _day   = rubrica_io_get_prop(node, "day", err);
  _month = rubrica_io_get_prop(node, "month", err);
  _year  = rubrica_io_get_prop(node, "year", err);
  
  if (_day && _month && _year)
    {
      if ((g_ascii_strcasecmp(_day,   "BadDay") == 0) ||
	  (g_ascii_strcasecmp(_day,   "") == 0))
	goto exit;
      if ((g_ascii_strcasecmp(_month, "BadMonth") == 0) ||
	  (g_ascii_strcasecmp(_month, "") == 0))
	goto exit;	
      if ((g_ascii_strcasecmp(_year,  "BadYear")  == 0) ||
	  (g_ascii_strcasecmp(_year,  "") == 0))
	goto exit;
    
      if (day)
	*day   = g_strdup(_day);
      if (month)
	*month = g_strdup(_month);
      if (year)
	*year  = g_strdup(_year);

      gdate = g_date_new_dmy(atoi(_day), atoi(_month), atoi(_year));
      g_date_to_struct_tm(gdate, &tm);
      g_date_free(gdate);

      *err = NO_ERROR;
      return mktime(&tm);
    }

 exit:
  *err = UNREADABLE_CALENDAR;
  if (day)
    *day  = "";
  if (month)
    *month = "";
  if (year)
    *year = "";

  if (_day)   
    g_free(_day);
  if (_month) 
    g_free(_month);
  if (_year)  
    g_free(_year);

  return (time_t) -1;
}


gchar* 
rubrica_io_get_prop(xmlNodePtr node, gchar* key, RubricaError *err)
{
  gchar* ret = NULL;

  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, NULL);

  *err = KEY_NOT_VALID;
  g_return_val_if_fail(key  != NULL, NULL);

  if (xmlHasProp(node, (xmlChar*) key))
    {
      *err = NO_ERROR;
//      ret  = g_strdup(xmlGetProp(node, (xmlChar*) key));
      ret  = xmlGetProp(node, (xmlChar*) key);

      return ret;
    }
  
  *err = UNREADABLE_PROPERTY;
  return NULL;
}


gchar*    
rubrica_io_get_prop_from (xmlNodePtr node, gchar* node_name,
			  gchar* key, RubricaError* err) 
{
  xmlNodePtr child;
  
  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, NULL);
  
  child = node->children;
  if (xmlIsBlankNode(child))
    child = child->next;   
  
  while (child)
    {
      if (xmlStrcmp(child->name, node_name) == 0)
	return rubrica_io_get_prop(child, key, err);
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
    }
  
  *err = UNREADABLE_NODE;
  return NULL;  
}


gboolean  
rubrica_io_get_bool_from (xmlNodePtr node, gchar* node_name,
			  gchar* key, RubricaError* err)
{
  xmlNodePtr child;
  
  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, FALSE);
  
  child = node->children;
  if (xmlIsBlankNode(child))
    child = child->next;   
  
  while (child)
    {
      if (xmlStrcmp(child->name, node_name) == 0)
	return rubrica_io_get_bool(child, key, err);
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
    }
  
  *err = UNREADABLE_NODE;
  return FALSE;
}


time_t   
rubrica_io_get_calendar_from (xmlNodePtr node, gchar* node_name,
			      gchar** day, gchar** month, gchar** year,
			      RubricaError *err)
{
  xmlNodePtr child;
  
  *err = UNREADABLE_NODE;
  g_return_val_if_fail(node != NULL, -1);
  
  child = node->children;
  if (xmlIsBlankNode(child))
    child = child->next;   
  
  while (child)
    {
      if (xmlStrcmp(child->name, node_name) == 0)
	return rubrica_io_get_calendar(child, day, month, year, err);
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
    }
  
  if (day)
    *day   = "BadDay";
  if (month)
    *month = "BadMonth";
  if (year)
    *year  = "BadYear";
  *err  = UNREADABLE_NODE;

  return (time_t) -1;  
}



xmlNodePtr 
rubrica_io_get_node(xmlNodePtr node, xmlChar* key)
{
  xmlNodePtr child;
  
  if(xmlIsBlankNode(node))
    node = node->next;

  if (xmlStrcmp(key, "Card") == 0) 
    {
      return node;
    }
  
  child = node->children;
  if(xmlIsBlankNode(child))
    child = child->next;
  
  while (child)
    {
      if (xmlStrcmp(child->name, key) == 0)
	return child;
      
      child = child->next;
      if (xmlIsBlankNode(child))
	child = child->next;
    }    
  
  return NULL;
}

