/*
*  RAL -- Rubrica Addressbook Library
*  file: load.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <time.h>
#include <libintl.h>
#include <glib.h>
#include <libxml/tree.h>

#include "load.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "contact.h"
#include "address.h"
#include "groups.h"
#include "net.h"
#include "notes.h"
#include "telephone.h"
#include "io.h"
#include "error.h"

#define _(x) gettext(x)

/*
  RubricaDoc file structure:
     version:    rubrica's version that made the addressbook file
     fileformat: #4
     doctype:    addressbook

  <?xml version="1.0" encoding="UTF-8"?>
  <Rubrica version="" fileformat="" doctype="">        root
     <Card                                             root child 
            name=""                                    the card name
            deletable=""                               is a deletable card?
            deleted=""                                 was card deleted?
            rank=""                                    card rank (1..5)
            id="pcxxx"                                 card id   (es:  pc643)
	    created=""                                 creation date
            last_change=""                             last change date
	    type="">                                   card's type 
         <Groups>                                      card child 
             <Group>...................</Group>        groups children
	     ...
         </Groups>       
	 <Addresses>                                   card child
             <Address type="">                         addresses children
                 <Street number="">....</Street>       address child
	         <City zip="">.........</City>
	         <Province>............</Province>
	         <State>...............</State>
	         <Country>.............</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">..........</Uri>
	     <Uri type="email">........</Uri>
	     <Uri type="aim">..........</Uri>
	     <Uri type="msn">..........</Uri>
	     <Uri type="yahoo">........</Uri>
	     <Uri type="icq">..........</Uri>
	     <Uri type="jabber">.......</Uri>	     
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="home">...</Telephone>
	     <Telephone type="work">...</Telephone>
	     <Telephone type="fax">....</Telephone>
	     ...
 	 </TelephoneNumbers>
         <Data>                                        card child
             <FirstName>...............</FirstName>    data child 
	     <MiddleName>..............</MiddleName>     "    "
	     <LastName>................</LastName>                    
	     <Nickname>................</NickName>                    
	     <Profession>..............</Profession>                  
 	     <NamePrefix>..............</NamePrefix>                  
	     <Title>...................</Title>                       
	     <Photo type="">...........</Photo>
	     <Birthday known="" day="" month="" year=""/>  
         </Data>
	 <Work>
	     <Assigment>...............</Assigment>
	     <Organization>............</Organization>
             <Department>..............</Department>
	     <SubDepartment>...........</SubDepartment>
	     <ManagerName>.............</ManagerName>
	     <ManagerPhone>............</ManagerPhone>
	     <CollabratorName>.........</CollabratorName>
	     <CollaboratorPhone>.......</CollabratorPhone>
         </Work>
	 <Notes married="">
             <SpouseName>..............</SpouseName>
	     <Child>...................</Child>
	     <OtherNotes>..............</OtherNotes>
	     <PublicKey>...............</PublicKey>
	     <SpouseBirthday known="" day="" month="" year=""/>
	     <Anniversary known="" day="" month="" year""/>
         </Notes>
     </Card>
     <Card>
     <Card                                             root child 
            name=""                                    the card name
            deletable=""                               is a deletable card?
            deleted=""                                 was card deleted?
            rank=""                                    card rank (1..5)
            id="ccxxx"                                 card id   (es: cc1039)
	    created=""                                 creation date
            last_change=""                             last change date
	    type="company">                            card's type 
         <Groups>                                      card child 
             <Group>...................</Group>        groups children
	     ...
         </Groups>       
         <Company>
             <CompanyName>.............</CompanyName>
	     <Notes>...................</Notes>
         </Company>
	 <Addresses>                                   card child
             <Address type="">                         addresses children
                 <Street number="">....</Street>       address child
	         <City zip="">.........</City>
	         <Province>............</Province>
	         <State>...............</State>
	         <Country>.............</Country>
 	      </Address>
	     ...
         </Addresses>
	 <Net>
	     <Uri type="web">..........</Uri>
	     <Uri type="email">........</Uri> 
	     ...
	 </Net>
	 <TelephoneNumbers>
	     <Telephone type="green">......</Telephone>
	     <Telephone type="operator">...</Telephone>
	     <Telephone type="fax">........</Telephone>
	     ...
 	 </TelephoneNumbers>
	 <CompanyContacts>
	     <Contact ref_id="id_personal">Nome Cognome</Contact>
	     ...
	 <CompanyContacts>
     </Card>
     ...
  </Rubrica>


  Rubrica
     |
     | children
     |
     v   next         next         next 
   Card-------->Card-------->....------->Card
     |
     | children
     |
     v   next                
   Groups--->Data--->Addresses--->Net--->....Phone-->Work-->Notes
     |        \          |         |           |        |      |      
     | child  | child    | child   |           |        |      |     
     |        |          v         v           v        v      v     
     |        |         ...       ...         ...      ...     ...    ...   
     v        v
   Group    FirstName--->MiddleName--->...

   pcxxx => personal card #xxx
   ccxxx => company  card #xxx
*/



gchar* rubrica_load_get_card_name     (xmlNodePtr node, RubricaError *err);

void   rubrica_load_load_card_data    (RubricaCard *card, xmlNodePtr node);
void   rubrica_load_load_company_data (RubricaCompanyCard* card, 
				       xmlNodePtr node);

void   rubrica_load_load_groups_data  (RubricaCard *card, xmlNodePtr node);
void   rubrica_load_load_contact_data (RubricaPersonalCard *card, 
				       xmlNodePtr node);
void   rubrica_load_load_address_data (RubricaCard *card, xmlNodePtr node);
void   rubrica_load_load_net_data     (RubricaCard *card, xmlNodePtr node);
void   rubrica_load_load_work_data    (RubricaPersonalCard *card,
				       xmlNodePtr node);
void   rubrica_load_load_note_data    (RubricaPersonalCard *card, 
				       xmlNodePtr node);
void   rubrica_load_load_phone_data   (RubricaCard *card, xmlNodePtr node);




gchar* 
rubrica_load_get_card_name(xmlNodePtr node, RubricaError *err)
{
  xmlChar* ret = NULL;

  ret = rubrica_io_get_prop(node, "name", err);

  if (!ret)
    {
//      rubrica_log_write(err);

      return NULL;
    }

  return ret;
}


void 
rubrica_load_load_card_data(RubricaCard *card, xmlNodePtr node)
{
  RubricaCardClass* klass;
  xmlNodePtr xmlcard = NULL;
  RubricaError err;
  gchar *tmp = NULL;  
  gchar* id;
  time_t t;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmlcard = rubrica_io_get_node(node, "Card");

  if (xmlcard)
    {
      gboolean bool;

      tmp = rubrica_load_get_card_name(xmlcard, &err);
      rubrica_card_set_card_name(card, tmp);
      g_free(tmp);

      bool = rubrica_io_get_bool(xmlcard, "deletable", &err);
      rubrica_card_set_deletable(card, bool);

      bool = rubrica_io_get_bool(xmlcard, "deleted", &err);
      rubrica_card_set_deleted(card, bool);

      tmp = rubrica_io_get_prop(xmlcard, "rank", &err);      
      if (tmp)
	{
	  rubrica_card_set_rank(card, (RubricaRank) atoi(tmp));
	  g_free(tmp);
	}
      else 
	rubrica_card_set_rank(card, (RubricaRank) 2);
      
      id = rubrica_io_get_prop(xmlcard, "id", &err);
      if (g_ascii_strcasecmp(id, "0") != 0)
	{
	  gchar* type = NULL;

	  klass = RUBRICA_CARD_GET_CLASS(card);
	  type = rubrica_io_get_prop(xmlcard, "type", &err);
	  
	  rubrica_card_set_id (card, id);
	  if (type && RUBRICA_CARD_CLASS(klass)->align_id)
	    RUBRICA_CARD_CLASS(klass)->align_id(card, id, type); 
	}
      g_free(id);
      
      t = rubrica_io_get_date(xmlcard, "created", &err);
      rubrica_card_set_creation_time(card, t);
    
      t = rubrica_io_get_date(xmlcard, "last_change", &err);
      rubrica_card_set_modification_time(card, t);
    }
  else
    {
      /* TODO 
	 
      segnalare l'errore, far scegliere all'utente se 
      continuare a caricare automaticamente il file 
      o avviare un caricamento assistito
      */	 
    }
}



void 
rubrica_load_load_groups_data(RubricaCard *card, xmlNodePtr node)
{
  RubricaGroups* groups = NULL;
  xmlNodePtr xmltablegroups = NULL;
  xmlNodePtr xmlgroup;
  RubricaError err;

  /*  
      <Groups>                            card child 
          <Group>.........</Group>        groups children
          ...
      </Groups>       
  */

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmltablegroups = rubrica_io_get_node(node, "Groups");
  
  if (xmltablegroups)
    {
      groups = rubrica_groups_new();

      if (!IS_RUBRICA_GROUPS(groups))
	return;
      
      xmlgroup = xmltablegroups->children;
      if (xmlIsBlankNode(xmlgroup))
	xmlgroup = xmlgroup->next;
      
      while (xmlgroup)
	{	  
	  gchar* group;

	  if (xmlIsBlankNode(xmlgroup))
	    xmlgroup = xmlgroup->next;		  
	  
	  group = rubrica_io_get_content(xmlgroup, &err);
	  rubrica_groups_add_group(groups, group);
	  
	  xmlgroup = xmlgroup->next;
	  if (xmlIsBlankNode(xmlgroup))
	    xmlgroup = xmlgroup->next;
	} 
     
      rubrica_card_set_groups(card, groups);
    }
}



void 
rubrica_load_load_contact_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  RubricaContact* contact;
  xmlNodePtr xmldata = NULL;
  time_t t;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  contact = rubrica_contact_new();
  if (!IS_RUBRICA_CONTACT(contact))
    {
      card->contact = NULL;
      return;
    }

  xmldata = rubrica_io_get_node(node, "Data");
  
  if (xmldata)  
    {   
      gchar* tmp;

      tmp = rubrica_io_get(xmldata, "FirstName", &err);
      rubrica_contact_set_first_name(contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "MiddleName", &err);
      rubrica_contact_set_middle_name(contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "LastName", &err);
      rubrica_contact_set_last_name(contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "NickName", &err);
      rubrica_contact_set_nick_name (contact, tmp);
      g_free(tmp);

      rubrica_io_get(xmldata, "Profession", &err);
      rubrica_contact_set_profession (contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "NamePrefix", &err);
      rubrica_contact_set_prefix (contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "Title", &err);
      rubrica_contact_set_title (contact, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(xmldata, "Photo", &err);
      rubrica_contact_set_photo (contact, tmp);
      g_free(tmp);

      t = rubrica_io_get_calendar_from (xmldata, "Birthday", 
					NULL, NULL, NULL, &err);
      rubrica_contact_set_birthday (contact, t);

      rubrica_personal_card_set_contact(card, contact);
    }
  else 
    g_warning((_("May be this file is corrupted. Can't read %s field"), 
	       "Data"));  
}



void  
rubrica_load_load_address_data(RubricaCard *card, xmlNodePtr node)
{
  RubricaAddress* address = NULL;
  RubricaError err;
  xmlNodePtr xmladd = NULL;
  xmlNodePtr xmladdress;
//  xmlNodePtr child;
  gchar *tmp;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmladd = rubrica_io_get_node(node, "Addresses");

  if (xmladd)
    {
      RubricaAddressType type;

      /* child is the head of an <address> list */
      xmladdress = xmladd->children;  
      if (xmlIsBlankNode(xmladdress))
	xmladdress = xmladdress->next;
      
      while(xmladdress)
	{
	  address = rubrica_address_new();
	  
	  if (!IS_RUBRICA_ADDRESS(address))
	    {
	      g_warning("address obj get wrong type");

	      return;
	    }
	  
	  
	  /* The address's data:

	    <Address type="">                         addresses children
                <Street number="">....</Street>       address child 
		...
	    </Address>
	  */

	  /* get address type --- <Address type=""> 
	   */
	  tmp  = rubrica_io_get_prop(xmladdress, "type", &err);
	  type = rubrica_address_get_decoded_type(tmp);
	  rubrica_address_set_address_type(address, type);
	  g_free(tmp);

	  /* get address children --- <Street number="">....</Street>, etc 

	  child = xmladdress->children;  
	  if (xmlIsBlankNode(child))
	    child = child->next;
	  */
	  tmp = rubrica_io_get(xmladdress, "Street", &err);
	  rubrica_address_set_street(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get_prop_from(xmladdress, "Street", "number", &err);
	  rubrica_address_set_number(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get(xmladdress, "City", &err);
	  rubrica_address_set_city(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get_prop_from(xmladdress, "City", "zip", &err);
	  rubrica_address_set_zip(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get(xmladdress, "Province", &err);
	  rubrica_address_set_province(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get(xmladdress, "State", &err);
	  rubrica_address_set_state(address, tmp);
	  g_free(tmp);

	  tmp = rubrica_io_get(xmladdress, "Country", &err);
	  rubrica_address_set_country(address, tmp);
	  g_free(tmp);
	  
	  rubrica_card_add_address(card, address);

	  xmladdress = xmladdress->next;
	  if (xmlIsBlankNode(xmladdress)) 
	    xmladdress = xmladdress->next;
	}
    }
  else 
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Address"));
    }
}


void
rubrica_load_load_net_data(RubricaCard *card, xmlNodePtr node)
{
  RubricaNetAddress* net;
  RubricaError err;
  xmlNodePtr xmlnet_child;
  xmlNodePtr xmlnet = NULL;
  gchar* net_data;
  gchar* net_type;

  /*
    <Net>
       <Uri type="web">..........</Uri>
       <Uri type="email">........</Uri> 
       <Uri type="icq">..........</Uri>
       ...
    </Net>
  */
  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmlnet = rubrica_io_get_node(node, "Net");

  if (xmlnet)
    {
      xmlnet_child = xmlnet->children;
      if (xmlIsBlankNode(xmlnet_child))
	xmlnet_child = xmlnet_child->next;

      while (xmlnet_child)
	{
	  RubricaNetAddressType type;

	  if (xmlIsBlankNode(xmlnet_child))
	    xmlnet_child = xmlnet_child->next;
	  
	  net_data = rubrica_io_get_content(xmlnet_child, &err);
	  net_type = rubrica_io_get_prop(xmlnet_child, "type", &err);

	  if (net_data)
	    {
	      net = rubrica_net_address_new();
	      
	      if(!IS_RUBRICA_NET_ADDRESS(net))
		{
		  g_warning("net obj get wrong type");
		  
		  g_free(net_type);
		  g_free(net_data);

		  return;
		}
	      
	      rubrica_net_address_set_url(net, net_data);
	      type = rubrica_net_address_get_decoded_type(net_type);
	      rubrica_net_address_set_url_type(net, type);

	      rubrica_card_add_net_address(card, net);
	    }

	  xmlnet_child = xmlnet_child->next;
	  if (xmlIsBlankNode(xmlnet_child)) 
	    xmlnet_child = xmlnet_child->next;
	}
    }
  else
    {
      // errore
    }
}


/* 

void 
rubrica_io_load_web_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  xmlNodePtr child;
  RubricaCardNetUri* uri;
  xmlNodePtr xmlweb = NULL;
  gchar *tmp;
  
  xmlweb = rubrica_io_get_node(node, "WebAddresses");

  if (xmlweb)
    {
      child = xmlweb->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  tmp = rubrica_io_get_content(child, &err);
	  if (tmp)
	    {
	      uri = g_malloc (sizeof(RubricaCardNetUri));
	      
	      if (!uri)
		{
		  g_warning("Out of memory");
		  if (personal->net)
		    personal->net = g_list_append(personal->net, NULL);
		  
		  return;	      
		}

	      uri->url  = tmp;
	      uri->type = g_strdup("web");
	      
	      personal->net = g_list_append(personal->net, uri);
	    }
	  
	  child = child->next;
	}
    }
  else
    {
      // errore
    }
}



void 
rubrica_io_load_email_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  RubricaCardNetUri* uri;
  xmlNodePtr xmlemail = NULL;
  xmlNodePtr child;
  gchar *tmp;
  
  xmlemail = rubrica_io_get_node(node, "EmailAddresses");
  
  if (xmlemail)
    {
      child = xmlemail->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      while (child)
	{
	  if (xmlIsBlankNode(child))
	    child = child->next;

	  tmp = rubrica_io_get_content(child, &err);
	  if (tmp)
	    {
	      uri = g_malloc (sizeof(RubricaCardNetUri));
	      
	      if (!uri)
		{
		  g_warning("Out of memory");
		  if (personal->net)
		    personal->net = g_list_append(personal->net, NULL);
		  
		  return;	      
		}

	      uri->url  = tmp;
	      uri->type = g_strdup("email");
	      
	      personal->net = g_list_append(personal->net, uri);
	    }
	  
	  child = child->next;
	}
    }
  else
    {
      // errore
    }  

}

*/

void
rubrica_load_load_phone_data(RubricaCard *card, xmlNodePtr node)
{
  xmlNodePtr xmltel_child;
  xmlNodePtr xmltel = NULL;
  RubricaError err;
  RubricaTelephone* tel;
  gchar* number;
  gchar* type;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  xmltel = rubrica_io_get_node(node, "TelephoneNumbers");

  if (xmltel)
    {
      xmltel_child = xmltel->children;
      if (xmlIsBlankNode(xmltel_child))
	xmltel_child = xmltel_child->next;

      while(xmltel_child)
	{
	  if (xmlIsBlankNode(xmltel_child))
	    xmltel_child = xmltel_child->next;
	  
	  number = rubrica_io_get_content(xmltel_child, &err);
	  type   = rubrica_io_get_prop(xmltel_child, "type", &err);

	  if (number)
	    {
	      tel = rubrica_telephone_new();
	      
	      if(!IS_RUBRICA_TELEPHONE(tel))
		{
		  g_warning("telephone obj get wrong type");
		  
		  return;
		}
	      
	      rubrica_telephone_set_number(tel, number);
	      rubrica_telephone_set_type(tel, type);

	      rubrica_card_add_telephone(card, tel);

	      g_free(number);
	      g_free(type);
	    }
	  
	  xmltel_child = xmltel_child->next;
	  if (xmlIsBlankNode(xmltel_child)) 
	    xmltel_child = xmltel_child->next;
	} 
    }
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "TelephoneNumbers"));
    }
}


void 
rubrica_load_load_company_data (RubricaCompanyCard* card, xmlNodePtr node)
{
  RubricaError err;
  xmlNodePtr child;  
  xmlNodePtr xmlcompany = NULL;
  gchar* tmp;

  g_return_if_fail(IS_RUBRICA_COMPANY_CARD(card));

  xmlcompany = rubrica_io_get_node(node, "Company");

  if (xmlcompany)
    {
      child = xmlcompany->children;
      if (xmlIsBlankNode(child))
	child = child->next;
      
      tmp = rubrica_io_get(child, "CompanyName", &err);
      rubrica_company_card_set_company_name (card, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "PI", &err);
      rubrica_company_card_set_pi(card, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "Notes", &err);
      rubrica_company_card_set_notes (card, tmp);
      g_free(tmp);
    }
  else   
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Company"));
    }
}


void 
rubrica_load_load_work_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  xmlNodePtr xmlwork = NULL;
  xmlNodePtr child;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  xmlwork = rubrica_io_get_node(node, "Work");

  if (xmlwork)
    {     
      RubricaWork* work;
      gchar* tmp;

      child = xmlwork->children;
      if (xmlIsBlankNode(child))
	child = child->next;

      work = rubrica_work_new();
      
      if (!IS_RUBRICA_WORK(work))
	return;

      tmp = rubrica_io_get(child, "Assigment", &err);      
      rubrica_work_set_assigment(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "Organization", &err);
      rubrica_work_set_organization(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "Department", &err);
      rubrica_work_set_department(work, tmp);
      g_free(tmp);
      
      tmp = rubrica_io_get(child, "SubDepartment", &err);
      rubrica_work_set_subdepartment(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "ManagerName", &err);
      rubrica_work_set_manager(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "ManagerPhone", &err);
      rubrica_work_set_mtelephone(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "CollaboratorName", &err);
      rubrica_work_set_collaborator(work, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "CollaboratorPhone", &err);
      rubrica_work_set_ctelephone(work, tmp);
      g_free(tmp);

      rubrica_personal_card_set_work(card, work);
    }
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Work"));
    }
}


void 
rubrica_load_load_note_data(RubricaPersonalCard *card, xmlNodePtr node)
{
  RubricaError err;
  xmlNodePtr xmlnote = NULL;
  xmlNodePtr child;

  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(card));

  xmlnote = rubrica_io_get_node(node, "Notes");
  
  if (xmlnote)
    {      
      RubricaNotes* notes;
      gchar* tmp;
      gboolean bool;
      time_t t;

      notes = rubrica_notes_new();

      if (!IS_RUBRICA_NOTES(notes))
	return;

      bool = rubrica_io_get_bool(xmlnote, "married", &err);
      rubrica_notes_set_married (notes, bool);
      
      child = xmlnote->children;
      if (xmlIsBlankNode(child))
	child = child->next;     

      tmp = rubrica_io_get(child, "SpouseName", &err);
      rubrica_notes_set_spouse_name(notes, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "Child", &err);
      rubrica_notes_set_children_notes(notes, tmp);
      g_free(tmp);


      tmp = rubrica_io_get(child, "OtherNotes", &err);
      rubrica_notes_set_other_notes(notes, tmp);
      g_free(tmp);

      tmp = rubrica_io_get(child, "PublicKey", &err);
      rubrica_notes_set_public_key(notes, tmp);
      g_free(tmp);

      bool = rubrica_io_get_bool_from(child, "SpouseBirthday", "known", &err);
      t    = rubrica_io_get_calendar_from (child, "SpouseBirthday",
					   NULL, NULL, NULL, &err);
      if (bool)
	{
	  rubrica_notes_set_know_birthday(notes, TRUE);
	  rubrica_notes_set_birthday (notes, t);
	}

      bool = rubrica_io_get_bool_from(child, "Anniversary", "known", &err);
      t    = rubrica_io_get_calendar_from (child, "Anniversary",
					   NULL, NULL, NULL, &err);
      if (bool)
	{
	  rubrica_notes_set_know_anniversary(notes, TRUE);
	  rubrica_notes_set_anniversary(notes, t); 
	}

      rubrica_personal_card_set_notes (card, notes);
    } 
  else
    {
      g_warning((_("May be this file is corrupted. Can't read %s field"), 
		 "Notes"));
    }
}




/*    Public
*/

/*  open the file "filename"
    return ptr to xml doc or NULL on error
 */
xmlDocPtr 
rubrica_load_open_doc(gchar *filename, RubricaError* err)
{
  xmlDocPtr doc;

  *err = NO_FILENAME;
  g_return_val_if_fail(filename != NULL, NULL);
  
  /* read and parse file 
   */
  doc = xmlParseFile(filename);

  if (!doc)
    {
      //      rubrica_dialog_error_loading_file(filename, RE_PARSE_XML_DOC);
      *err = PARSING_DOC;
      
      return NULL;
    }  

  *err = NO_ERROR;
  return doc;
}


/*  get the file format of the rubrica's doc    
*/
gint 
rubrica_load_get_fileformat(xmlDocPtr doc, RubricaError* err)
{
  gchar* tmp;
  gint ret;

  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, -1);

  if(!xmlHasProp(doc->children, "fileformat"))
    {
      *err = UNKNOWN_FILEFORMAT;
      return 0;    
    }

  *err = NO_ERROR;
  tmp = xmlGetProp(doc->children, "fileformat");
  ret = atoi(tmp);
  xmlFree(tmp);

  return ret;
}


/*  get version.
    the version of rubrica that make the file
*/
gchar*
rubrica_load_get_version(xmlDocPtr doc, RubricaError *err)
{
  gchar* ret = NULL;
  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, NULL);
  
  if(!xmlHasProp(doc->children, "version"))
    {
      *err = UNKNOWN_VERSION;
      return NULL;
    }

  *err = NO_ERROR;
  ret = xmlGetProp(doc->children, "version");

  return ret;
}


/*  get the document type
    values: ADDRESSBOOK_TYPE_DOC
            ACCOUNT_TYPE_DOC         // removed, may be added in future
*/
gchar*
rubrica_load_get_document_tpye(xmlDocPtr doc, RubricaError *err)
{
  gchar* ret = NULL;

  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, NULL);
  
  if(!xmlHasProp(doc->children, "doctype"))
    {
      *err = UNKNOWN_DOC;
      return NULL;
    }

  *err = NO_ERROR;
  ret  = xmlGetProp(doc->children, "doctype");

  return ret;
}


/*  check if file "filename" is a rubrica doc
    return TRUE if xml doc is a rubrica doc FALSE otherwise
*/
gboolean 
rubrica_load_is_rubrica_doc(xmlDocPtr doc, RubricaError *err)
{
  *err = NO_RUBRICA_DOC;
  g_return_val_if_fail(doc != NULL, FALSE);

  if (!doc->children || 
      !doc->children->name || 
      g_ascii_strcasecmp(doc->children->name, "Rubrica") != 0)
    { 
      *err = UNKNOWN_DOC;
//      rubrica_dialog_error_loading_file(fname, RE_NO_RUBRICA_HEADER);  

      return FALSE;
    }  

  *err = NO_ERROR;

  return TRUE;
}


RubricaPersonalCard* 
rubrica_load_read_personal_xmlcard(xmlNodePtr xmlcard)
{
  RubricaPersonalCard* card;
  
  card = rubrica_personal_card_new();
  
  rubrica_load_load_card_data(RUBRICA_CARD(card), xmlcard); 
  rubrica_load_load_groups_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_contact_data(card, xmlcard);
  rubrica_load_load_address_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_net_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_phone_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_work_data(card, xmlcard);
  rubrica_load_load_note_data(card, xmlcard);

  return card;
}


RubricaPersonalCard* 
rubrica_load_read_old_personal_xmlcard(xmlNodePtr xmlcard)
{
  RubricaPersonalCard* card = NULL;

  /*
  card = rubrica_personal_card_new();
  
  rubrica_load_load_card_data(RUBRICA_CARD(card), xmlcard); 
  rubrica_load_load_groups_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_contact_data(card, xmlcard);
  rubrica_load_load_address_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_net_data(RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_phone_data(RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_work_data(card, xmlcard);
  rubrica_load_load_note_data(card, xmlcard);
  */
  return card;
}


RubricaCompanyCard *
rubrica_load_read_company_xmlcard(xmlNodePtr xmlcard)
{
  RubricaCompanyCard *card;

  card = rubrica_company_card_new();

  rubrica_load_load_card_data (RUBRICA_CARD(card), xmlcard);
  rubrica_load_load_groups_data (RUBRICA_CARD(card), xmlcard);  

  rubrica_load_load_company_data (card, xmlcard);
  rubrica_load_load_address_data (RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_net_data (RUBRICA_CARD(card), xmlcard);  
  rubrica_load_load_phone_data (RUBRICA_CARD(card), xmlcard);

  return card;
}



/**
 **
 **  Load Routines
 **
 **/
/*
gboolean
rubrica_load_load_xmlfile(gchar *filename, RubricaErrorType* err)
{
  RubricaDoc *doc;
  gchar *fpath, *fname;
  


  rubrica_add_page_view(filename); 
  doc = rubrica_doc_get_current_document();
*/
  /* real file load 
   */
/*
  if (!rubrica_load_real_load_xmlfile(doc, filename, err))
    { 
      return FALSE;
    }
*/
  /* if we are here, the file has been loaded, 
     now we need to put in memory some information 
     about this file and addressbook      
  */
/*
  fpath = g_path_get_dirname(filename);
  fname = g_path_get_basename(filename);
  
  rubrica_doc_set_path(doc, fpath);
  rubrica_doc_set_name(doc, fname);
  rubrica_doc_set_new(doc, FALSE);
  rubrica_doc_set_empty(doc, FALSE);
  rubrica_doc_set_modified(doc, FALSE);

  g_free(fname);
  
  *err = RE_NO_ERROR;
  return TRUE;
}

*/

/* 
 *  read the addressbook file 
 */
/*
gboolean 
rubrica_load_real_load_xmlfile(RubricaIo* io, gchar *fname, 
				RubricaErrorType *err)
{
*/
  /* parse doc, extract address from file and put them in memory 
   */
//  rubrica_load_parse_doc(doc, xmldoc, err);
  
  /* correggere id */
//  rubrica_personal_test_id(doc);
  


  /* addressbook has been parsed. Free memory */
/*
  xmlFreeDoc(xmldoc);
  

  msg = g_strdup_printf(_("document %s succesfully opened"), fname);
  g_print("\n%s", msg);  
  g_free(msg);

  return TRUE;  
}
*/
