/*
*  RAL -- Rubrica Addressbook Library
*  file: net.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "net.h"
#include "utils.h"


typedef struct _NetConversion {
  gint   token;
  gchar* label;
} NetConversion;


struct _RubricaNetAddressPrivate {
  gboolean dispose_has_run;
};


static NetConversion table[] = {
  {RUBRICA_NET_ADDRESS_WEB,        "web"    },
  {RUBRICA_NET_ADDRESS_EMAIL,      "email"  },
  {RUBRICA_NET_ADDRESS_IRC_AIM,    "aim"    },
  {RUBRICA_NET_ADDRESS_IRC_ICQ,    "icq"    },
  {RUBRICA_NET_ADDRESS_IRC_JABBER, "jabber" },
  {RUBRICA_NET_ADDRESS_IRC_YAHOO,  "yahoo"  },
  {RUBRICA_NET_ADDRESS_IRC_MSN,    "msn"    },
  {RUBRICA_NET_ADDRESS_UNKNOWN,    "unknown"},
};



static void rubrica_net_address_class_init (RubricaNetAddressClass* klass);
static void rubrica_net_address_init       (RubricaNetAddress* obj);

static void rubrica_net_address_dispose    (RubricaNetAddress* obj);
static void rubrica_net_address_finalize   (RubricaNetAddress* obj);


GType
rubrica_net_address_get_type()
{
  static GType rubrica_net_address_type = 0;
  
  if (!rubrica_net_address_type)
    {
      static const GTypeInfo rubrica_net_address_info =
	{
	  sizeof(RubricaNetAddressClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_net_address_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaNetAddress),
	  0,
	  (GInstanceInitFunc) rubrica_net_address_init
	};

      rubrica_net_address_type = 
	g_type_register_static (G_TYPE_OBJECT, "RubricaNetAddress",
				&rubrica_net_address_info, 0);
    }
  
  return rubrica_net_address_type;
}


static void
rubrica_net_address_class_init(RubricaNetAddressClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_net_address_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_net_address_finalize;
}



static void
rubrica_net_address_init(RubricaNetAddress* self)
{
  self->url  = NULL;
  self->type = NULL;

  self->private = g_new(RubricaNetAddressPrivate, 1);
  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_net_address_dispose (RubricaNetAddress* self)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_net_address_finalize (RubricaNetAddress* self)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  rubrica_utils_free_string(self->url);
  rubrica_utils_free_string(self->type);

  g_free(self->private);
  self->private = NULL;
}


RubricaNetAddress*
rubrica_net_address_new(void)
{
  RubricaNetAddress* self;

  self = g_object_new(rubrica_net_address_get_type(), NULL);

  return self;
}


void
rubrica_net_address_free(RubricaNetAddress* self)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  g_object_unref(self);   
}



void   
rubrica_net_address_set (RubricaNetAddress *self, gchar* url, 
			 RubricaNetAddressType type)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  rubrica_net_address_set_url(self, url);
  rubrica_net_address_set_url_type(self, type);   
}


void   
rubrica_net_address_set_url (RubricaNetAddress *self, gchar* url)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  rubrica_utils_free_string(self->url);

  self->url = g_strdup(url);
}


void   
rubrica_net_address_set_url_type (RubricaNetAddress *self, 
				  RubricaNetAddressType type)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  rubrica_utils_free_string(self->type);
  self->type = g_strdup(table[type].label);
}


void   
rubrica_net_address_get(RubricaNetAddress *self, gchar** url, gchar** type)
{
  g_return_if_fail(IS_RUBRICA_NET_ADDRESS(self));

  if (url)
    *url = g_strdup(rubrica_net_address_get_url(self));
  if (type)
    *type = rubrica_net_address_get_url_type(self);
}


gchar* 
rubrica_net_address_get_url (RubricaNetAddress *self) 
{
  g_return_val_if_fail(IS_RUBRICA_NET_ADDRESS(self), NULL);

  return self->url;
}


gchar* 
rubrica_net_address_get_url_type (RubricaNetAddress *self)

{
  g_return_val_if_fail(IS_RUBRICA_NET_ADDRESS(self), NULL);

  return self->type;
}


RubricaNetAddress* 
rubrica_net_address_copy (RubricaNetAddress *self)
{
  RubricaNetAddress* new;
  RubricaNetAddressType type;
  gchar* tmp;

  g_return_val_if_fail(IS_RUBRICA_NET_ADDRESS(self), NULL);

  new = rubrica_net_address_new();

  tmp  = rubrica_net_address_get_url_type(self);
  type = rubrica_net_address_get_decoded_type(tmp);

  rubrica_net_address_set(new, rubrica_net_address_get_url(self), type);

  return new;
}


RubricaNetAddressType 
rubrica_net_address_get_decoded_type (gchar* type)
{
  RubricaNetAddressType ret = RUBRICA_NET_ADDRESS_UNKNOWN;

  if (!type)
    return ret;

  if (g_ascii_strcasecmp(type, "web") == 0)
    ret = RUBRICA_NET_ADDRESS_WEB;
  else if (g_ascii_strcasecmp(type, "email") == 0)
    ret = RUBRICA_NET_ADDRESS_EMAIL;
  else if (g_ascii_strcasecmp(type, "aim") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_AIM;
  else if (g_ascii_strcasecmp(type, "jabber") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_JABBER;
  else if (g_ascii_strcasecmp(type, "icq") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_ICQ;
  else if (g_ascii_strcasecmp(type, "yahoo") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_YAHOO;
  else if (g_ascii_strcasecmp(type, "msn") == 0)
    ret = RUBRICA_NET_ADDRESS_IRC_MSN;
  
  return ret;
}

  /*
gchar* 
rubrica_net_address_encode_type(RubricaNetAddress *self) 
{
  RubricaNetAddressType type;

  type = rubrica_net_address_get_url_type(self);

  return table[type].label;
}
  */
