/*
*  RAL -- Rubrica Addressbook Library
*  file: notes.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "notes.h"
#include "date.h"
#include "utils.h"


struct _RubricaNotesPrivate {
  gboolean dispose_has_run;
};



static void rubrica_notes_class_init (RubricaNotesClass* klass);
static void rubrica_notes_init       (RubricaNotes* obj);

static void rubrica_notes_dispose    (RubricaNotes* obj);
static void rubrica_notes_finalize   (RubricaNotes* obj);


GType
rubrica_notes_get_type()
{
  static GType notes_type = 0;
  
  if (!notes_type)
    {
      static const GTypeInfo notes_info =
	{
	  sizeof(RubricaNotesClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_notes_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaNotes),
	  0,
	  (GInstanceInitFunc) rubrica_notes_init
	};

      notes_type = g_type_register_static (G_TYPE_OBJECT, "RubricaNotes",
					   &notes_info, 0);
    }
  
  return notes_type;
}


static void
rubrica_notes_class_init(RubricaNotesClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_notes_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_notes_finalize;
}


static void
rubrica_notes_init(RubricaNotes* self)
{
  self->married    = FALSE;
  self->spouse     = NULL;
  self->children   = NULL;
  self->notes      = NULL;
  self->pubkey     = NULL;
  self->birthday    = rubrica_date_new();
  self->anniversary = rubrica_date_new();
  
  self->private = g_new(RubricaNotesPrivate, 1);
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_notes_dispose (RubricaNotes* self)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  if (self->private->dispose_has_run)
    return;

  rubrica_date_free(self->birthday);
  rubrica_date_free(self->anniversary);

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_notes_finalize (RubricaNotes* self)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_utils_free_string(self->spouse);
  rubrica_utils_free_string(self->children);
  rubrica_utils_free_string(self->notes);
  rubrica_utils_free_string(self->pubkey);

  g_free(self->private);
  self->private = NULL;
}


RubricaNotes*
rubrica_notes_new(void)
{
  RubricaNotes* self;

  self = g_object_new(rubrica_notes_get_type(), NULL);

  return self;
}


void
rubrica_notes_free(RubricaNotes* self)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  g_object_unref(self);   
}



void 
rubrica_notes_set (RubricaNotes* self, gboolean is_married,
		   gboolean know_birtday, gboolean know_anniversary,
		   gchar* name, gchar* cnote, gchar* onote, 
		   gchar* pubkey, time_t birth, time_t anniversary)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_notes_set_married(self, is_married);
  rubrica_notes_set_know_birthday(self, know_birtday);
  rubrica_notes_set_know_anniversary(self, know_anniversary);
  rubrica_notes_set_spouse_name(self, name);
  rubrica_notes_set_children_notes(self, cnote);
  rubrica_notes_set_other_notes(self, onote);
  rubrica_notes_set_public_key(self, pubkey);
  rubrica_notes_set_birthday (self, birth);
  rubrica_notes_set_anniversary (self, anniversary);
}


void 
rubrica_notes_set_married (RubricaNotes* self, gboolean is_married)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  self->married = is_married;
}


void 
rubrica_notes_set_know_birthday (RubricaNotes* self, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));
  
  rubrica_date_know_date(self->birthday, know);;
}


void 
rubrica_notes_set_know_anniversary (RubricaNotes* self, gboolean know)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_date_know_date(self->anniversary, know);
}


void 
rubrica_notes_set_spouse_name (RubricaNotes* self, gchar* name)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_utils_free_string(self->spouse);
  self->spouse = g_strdup(name);
}


void 
rubrica_notes_set_children_notes (RubricaNotes* self, gchar* cnote)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_utils_free_string(self->children);
  self->children = g_strdup(cnote);
}


void 
rubrica_notes_set_other_notes (RubricaNotes* self, gchar* note)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));

  rubrica_utils_free_string(self->notes);
  self->notes = g_strdup(note);
}


void 
rubrica_notes_set_public_key (RubricaNotes* self, gchar* pubkey)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));
  
  rubrica_utils_free_string(self->pubkey);
  self->pubkey = g_strdup(pubkey);  
}

void 
rubrica_notes_set_birthday (RubricaNotes* self, time_t birth)
{
  rubrica_date_set_date(self->birthday, birth);
}


void 
rubrica_notes_set_anniversary (RubricaNotes* self, time_t anniversary)
{
  rubrica_date_set_date(self->anniversary, anniversary);
}


void     
rubrica_notes_get (RubricaNotes* self,  
		   gboolean* married, gboolean* birth, gboolean *ann,
		   gchar** spouse, gchar** child, gchar** notes, gchar** pkey,
		   gchar** bday, gchar** bmonth, gchar** byear,
		   gchar** aday, gchar** amonth, gchar** ayear)
{
  g_return_if_fail(IS_RUBRICA_NOTES(self));
  
  if (married)
    *married = rubrica_notes_is_married (self);
  if (birth) 
    *birth = rubrica_notes_known_birthday(self);
  if (ann)    
    *ann = rubrica_notes_know_anniversary(self);
  if (spouse)
    *spouse = rubrica_notes_get_spouse_name(self);
  if (child)
    *child = rubrica_notes_get_children_notes(self);
  if (notes) 
    *notes = rubrica_notes_get_other_notes(self);
  if (pkey)
    *pkey = rubrica_notes_get_public_key(self);
  if (bday) 
    *bday = rubrica_notes_get_birth_day(self);
  if (bmonth)
    *bmonth = rubrica_notes_get_birth_month(self);
  if (byear) 
    *byear = rubrica_notes_get_birth_year(self);
  if (aday)   
    *aday = rubrica_notes_get_anniversary_day(self);
  if (amonth) 
    *amonth = rubrica_notes_get_anniversary_month(self);
  if (ayear)  
    *ayear = rubrica_notes_get_anniversary_year(self);
}


gboolean 
rubrica_notes_is_married (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), FALSE);

  return self->married;
}


gboolean 
rubrica_notes_known_birthday (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), FALSE);

  return rubrica_date_is_date_known(self->birthday);
}


gboolean 
rubrica_notes_know_anniversary (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), FALSE);

  return rubrica_date_is_date_known(self->anniversary);
}


gchar*   
rubrica_notes_get_spouse_name (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return self->spouse;
}


gchar*   
rubrica_notes_get_children_notes (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return self->children;
}


gchar*   
rubrica_notes_get_other_notes (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return self->notes;
}


gchar*   
rubrica_notes_get_public_key (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return self->pubkey;
}


gchar*   
rubrica_notes_get_birth_day (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_day(self->birthday);
}


gchar*   
rubrica_notes_get_birth_month (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_month(self->birthday);
}


gchar*   
rubrica_notes_get_birth_year (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_year(self->birthday);
}


gchar*   
rubrica_notes_get_anniversary_day (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_day(self->anniversary);
}


gchar*   
rubrica_notes_get_anniversary_month (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_month(self->anniversary);
}


gchar*   
rubrica_notes_get_anniversary_year (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);
 
  return rubrica_date_get_year(self->anniversary);
}


time_t   
rubrica_notes_get_birthdate (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), (time_t) -1);
  
  return rubrica_date_get_date(self->birthday);
}


time_t   
rubrica_notes_get_anniversarydate (RubricaNotes* self)
{
  g_return_val_if_fail(IS_RUBRICA_NOTES(self), (time_t) -1);

  return rubrica_date_get_date(self->anniversary);
}


void rubrica_notes_append_other_notes (RubricaNotes* self, gchar* note)
{
  gchar* tmp = NULL;
  gchar* new;

  g_return_if_fail(IS_RUBRICA_NOTES(self));
  g_return_if_fail(note != NULL);
  
  tmp = rubrica_notes_get_other_notes(self);
  if (tmp && (g_ascii_strcasecmp(tmp, "") != 0))
    new = g_strdup_printf("%s\n%s", note, tmp);
  else
    new = g_strdup_printf("%s\n", note);

  rubrica_notes_set_other_notes(self, new);
}


RubricaNotes* 
rubrica_notes_copy (RubricaNotes* self)
{
  RubricaNotes* new;

  g_return_val_if_fail(IS_RUBRICA_NOTES(self), NULL);

  new = rubrica_notes_new();

  rubrica_notes_set(new, 
		    rubrica_notes_is_married(self),
		    rubrica_notes_known_birthday(self),
		    rubrica_notes_know_anniversary(self),
		    rubrica_notes_get_spouse_name(self),
		    rubrica_notes_get_children_notes(self),
		    rubrica_notes_get_other_notes(self),
		    rubrica_notes_get_public_key(self),
		    rubrica_notes_get_birthdate(self),
		    rubrica_notes_get_anniversarydate(self));

  return new;
}
