/*
*  RAL -- Rubrica Addressbook Library
*  file: personal.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "card.h"
#include "personal.h"
#include "contact.h"
#include "work.h"
#include "notes.h"
#include "utils.h"
#include "save.h"


struct _RubricaPersonalCardPrivate {
  gboolean dispose_has_run;
};



static void rubrica_personal_card_class_init (RubricaPersonalCardClass* klass);
static void rubrica_personal_card_init       (RubricaPersonalCard* obj);

static void rubrica_personal_card_dispose    (RubricaPersonalCard* obj);
static void rubrica_personal_card_finalize   (RubricaPersonalCard* obj);


static gint rubrica_personal_info            (RubricaPersonalCard* self);
static void rubrica_personal_card_free       (RubricaPersonalCard* self);
static void rubrica_personal_print           (RubricaPersonalCard* self);


RubricaPersonalCard* rubrica_personal_card_copy (RubricaPersonalCard* self);



static RubricaCardClass* parent_class;


GType
rubrica_personal_card_get_type()
{
  static GType personal_card_type = 0;
  
  if (!personal_card_type)
    {
      static const GTypeInfo personal_card_info =
	{
	  sizeof(RubricaPersonalCardClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_personal_card_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaPersonalCard),
	  0,
	  (GInstanceInitFunc) rubrica_personal_card_init
	};
      
      personal_card_type = g_type_register_static (RUBRICA_CARD_TYPE,
						   "RubricaPersonal",
						   &personal_card_info, 0);
    }
  
  return personal_card_type;
}


static void
rubrica_personal_card_class_init(RubricaPersonalCardClass* klass)
{
  GObjectClass *obj_class;
  
  parent_class             = RUBRICA_CARD_CLASS(klass);
  parent_class->free       = rubrica_personal_card_free;
  parent_class->copy       = rubrica_personal_card_copy;
  parent_class->write_node = rubrica_save_write_personal_data;

  parent_class->info_type  = rubrica_personal_info;
  parent_class->print_card = rubrica_personal_print;

  obj_class = G_OBJECT_CLASS (klass);
  obj_class->dispose  = (GObjectFinalizeFunc) rubrica_personal_card_dispose;
  obj_class->finalize = (GObjectFinalizeFunc) rubrica_personal_card_finalize;
}


static void
rubrica_personal_card_init(RubricaPersonalCard* self)
{
  self->contact   = NULL;
  self->work      = NULL;
  self->notes     = NULL;

  self->private = g_new(RubricaPersonalCardPrivate, 1);
  self->private->dispose_has_run = FALSE;  
}



static void 
rubrica_personal_card_dispose (RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  
  if (self->private->dispose_has_run)
    return;
  
  /*    free the objects owned by personal card
   */
  rubrica_contact_free(RUBRICA_CONTACT(self->contact));
  rubrica_work_free(RUBRICA_WORK(self->work));
  rubrica_notes_free(RUBRICA_NOTES(self->notes));  

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_personal_card_finalize (RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  
  /*    finally, delete personal card's private structure
   */
  g_free(self->private);
  self->private = NULL;
}


static gint
rubrica_personal_info(RubricaPersonalCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(self), UNKNOW_CARD);

  return PERSONAL_CARD;
}


static void
rubrica_personal_card_free(RubricaPersonalCard* self)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));

  g_object_unref(G_OBJECT(self));
}



/*   Public
 */
RubricaPersonalCard*
rubrica_personal_card_new(void)
{
  RubricaPersonalCard* self;
  
  gulong id = 0L;
  gchar* pc_id = NULL;

  self = g_object_new(rubrica_personal_card_get_type(), NULL);

  id = RUBRICA_CARD_CLASS(parent_class)->generate_id(RUBRICA_CARD(self), 
						     "personal");
  if (id == 0L)
    return NULL;

  pc_id = g_strdup_printf("pc%ld", id); 
  rubrica_card_set_id(RUBRICA_CARD(self), pc_id); 
  
  g_free(pc_id);
  pc_id = NULL;
  
  return self;
}


void 
rubrica_personal_card_set_contact (RubricaPersonalCard* self, 
				   RubricaContact* contact)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  g_return_if_fail(IS_RUBRICA_CONTACT(contact));

  if (self->contact)
    rubrica_contact_free(RUBRICA_CONTACT(self->contact));

  self->contact = contact;
}


void 
rubrica_personal_card_set_work (RubricaPersonalCard* self,
				RubricaWork* work)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  g_return_if_fail(IS_RUBRICA_WORK(work));

  if (self->work)
    rubrica_work_free(RUBRICA_WORK(self->work));

  self->work = work;
}


void 
rubrica_personal_card_set_notes (RubricaPersonalCard* self, 
				 RubricaNotes* notes)
{
  g_return_if_fail(IS_RUBRICA_PERSONAL_CARD(self));
  g_return_if_fail(IS_RUBRICA_NOTES(notes));

  if (self->notes)
    rubrica_notes_free(RUBRICA_NOTES(self->notes));

  self->notes = notes;
}



RubricaContact*      
rubrica_personal_card_get_contact (RubricaPersonalCard* self)
{  
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(self), NULL);

  return self->contact;
}



RubricaWork*       
rubrica_personal_card_get_work (RubricaPersonalCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(self), NULL);

  return self->work;
}


RubricaNotes*      
rubrica_personal_card_get_notes (RubricaPersonalCard* self)
{
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(self), NULL);

  return self->notes;
}


RubricaPersonalCard* 
rubrica_personal_card_copy (RubricaPersonalCard* self)
{
  RubricaPersonalCard* new;
  RubricaContact* contact = NULL;
  RubricaWork* work = NULL;
  RubricaNotes* notes = NULL;
  
  g_return_val_if_fail(IS_RUBRICA_PERSONAL_CARD(self), NULL);

  new = rubrica_personal_card_new();

  //  rubrica_card_real_copy(RUBRICA_CARD(new), RUBRICA_CARD(self));

  contact = rubrica_contact_copy(rubrica_personal_card_get_contact(self));
  work    = rubrica_work_copy(rubrica_personal_card_get_work(self));
  notes   = rubrica_notes_copy(rubrica_personal_card_get_notes(self));
  
  rubrica_personal_card_set_contact(new, contact);
  rubrica_personal_card_set_work(new, work);
  rubrica_personal_card_set_notes(new, notes);

  return new;
}






static void 
rubrica_personal_print (RubricaPersonalCard* card)
{
  gpointer data;

  printf("\n\n*** Printing Personal Card data ***");
  printf("\ncard id: %s", rubrica_card_get_id(RUBRICA_CARD(card)));
  printf("\nname: %s\t rank: %d\tdeletable: %d\tdeleted: %d",
	 rubrica_card_get_card_name(RUBRICA_CARD(card)),
	 rubrica_card_get_rank(RUBRICA_CARD(card)),
	 rubrica_card_is_deletable(RUBRICA_CARD(card)),
	 rubrica_card_is_deleted(RUBRICA_CARD(card)));  
  printf("\ncard creation time: %ld\tmodification time: %ld", 
	 rubrica_card_get_creation_time(RUBRICA_CARD(card)),
	 rubrica_card_get_modification_time(RUBRICA_CARD(card)));

  printf("\n\n****** Groups");
  data = rubrica_card_get_group(RUBRICA_CARD(card));
  for(; data; data = rubrica_card_get_next_group(RUBRICA_CARD(card)))    
    printf("\ncard belongs to group: %s", (gchar*) data);

    
  printf("\n\n****** Addresses");
  data = rubrica_card_get_address(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_address(RUBRICA_CARD(card)))
    {
      printf("\naddress type: %s", 
	     rubrica_address_get_address_type(RUBRICA_ADDRESS(data))); 
      printf("\nstreet: %s\tnumber: %s", 
	     rubrica_address_get_street(RUBRICA_ADDRESS(data)), 
	     rubrica_address_get_number(RUBRICA_ADDRESS(data))); 
      printf("\ncity: %s\tzip: %s", 
	     rubrica_address_get_city(RUBRICA_ADDRESS(data)), 
	     rubrica_address_get_zip(RUBRICA_ADDRESS(data))); 
      printf("\nprovince: %s\tstate: %s\tcountry: %s", 
	     rubrica_address_get_province(RUBRICA_ADDRESS(data)),
	     rubrica_address_get_state(RUBRICA_ADDRESS(data)),
	     rubrica_address_get_country(RUBRICA_ADDRESS(data)));      
    }

  printf("\n\n****** Personal Data");
  printf("\nfirst: %s\tmiddle: %s\tlast: %s\tnick: %s", 
	 rubrica_contact_get_first_name(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_middle_name(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_last_name(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_nick_name(RUBRICA_CONTACT(card->contact))); 
  printf("\nprefix: %s\tprofession: %s\ttitle: %s", 
	 rubrica_contact_get_prefix(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_profession(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_title(RUBRICA_CONTACT(card->contact))); 
  printf("\nbirthday: %s/%s/%s", 
	 rubrica_contact_get_birth_day(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_birth_month(RUBRICA_CONTACT(card->contact)),
	 rubrica_contact_get_birth_year(RUBRICA_CONTACT(card->contact)));
  printf("\ncard photo: %s", 
	 rubrica_contact_get_photo(RUBRICA_CONTACT(card->contact)));

  printf("\n\n****** Net Addresses");
  data = rubrica_card_get_net_address (RUBRICA_CARD(card)); 
  for (; data; data = rubrica_card_get_next_net_address (RUBRICA_CARD(card)))
    {
      gchar* type;

      type = rubrica_net_address_get_url_type(RUBRICA_NET_ADDRESS(data));
      
      printf("\nurl: %s (url type: %s, url type code %d)", 
	     rubrica_net_address_get_url(RUBRICA_NET_ADDRESS(data)), 
	     type, rubrica_net_address_get_decoded_type(type)); 
    }

  printf("\n\n****** Telephone");
  data = rubrica_card_get_telephone(RUBRICA_CARD(card)); 
  for (; data; data = rubrica_card_get_next_telephone(RUBRICA_CARD(card)))
    printf("\ntelephone number (%s): %s",
	   rubrica_telephone_get_tel_type(RUBRICA_TELEPHONE(data)),
	   rubrica_telephone_get_number(RUBRICA_TELEPHONE(data))); 
   
  printf("\n\n****** Work");
  printf("\nassigment: %s", 
	 rubrica_work_get_assigment(RUBRICA_WORK(card->work))); 
  printf("\norganizzation: %s\tdepartment: %s\tsubdepartment: %s",
	 rubrica_work_get_organization(RUBRICA_WORK(card->work)),
	 rubrica_work_get_department(RUBRICA_WORK(card->work)),
	 rubrica_work_get_subdepartment(RUBRICA_WORK(card->work)));
  printf("\nmanager: %s\ttelephone: %s", 
	 rubrica_work_get_manager(RUBRICA_WORK(card->work)),
	 rubrica_work_get_mtelephone(RUBRICA_WORK(card->work)));  
  printf("\nsecretary: %s\ttelephone: %s", 
	 rubrica_work_get_collaborator(RUBRICA_WORK(card->work)),
	 rubrica_work_get_ctelephone(RUBRICA_WORK(card->work)));
  
  printf("\n\n****** Notes");
  printf("\ncontact is married: %d\tknow spouse birthday: %d"
	 "\tknow contact anniversary: %d",
	 rubrica_notes_is_married(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_known_birthday(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_know_anniversary(RUBRICA_NOTES(card->notes)));
  printf("\nspouse name: %s", 
	 rubrica_notes_get_spouse_name(RUBRICA_NOTES(card->notes)));
  printf("\nchildren: %s", 
	 rubrica_notes_get_children_notes(RUBRICA_NOTES(card->notes)) );
  printf("\nnotes: %s", 
	 rubrica_notes_get_other_notes(RUBRICA_NOTES(card->notes)));
  printf("\npublic key: %s", 
	 rubrica_notes_get_public_key(RUBRICA_NOTES(card->notes)));
  printf("\nspouse birthday: %s/%s/%s", 
	 rubrica_notes_get_birth_day(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_get_birth_month(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_get_birth_year(RUBRICA_NOTES(card->notes)));
  printf("\nanniversary day: %s/%s/%s", 
	 rubrica_notes_get_anniversary_day(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_get_anniversary_month(RUBRICA_NOTES(card->notes)),
	 rubrica_notes_get_anniversary_year(RUBRICA_NOTES(card->notes)));
}
