/*
*  RAL -- Rubrica Addressbook Library
*  file: telephone.h
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "telephone.h"
#include "utils.h"


struct _RubricaTelephonePrivate {
  gboolean dispose_has_run;
};


static void rubrica_telephone_class_init (RubricaTelephoneClass* klass);
static void rubrica_telephone_init       (RubricaTelephone* obj);

static void rubrica_telephone_dispose    (RubricaTelephone* obj);
static void rubrica_telephone_finalize   (RubricaTelephone* obj);


GType
rubrica_telephone_get_type()
{
  static GType tel_num_type = 0;
  
  if (!tel_num_type)
    {
      static const GTypeInfo tel_num_info =
	{
	  sizeof(RubricaTelephoneClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_telephone_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaTelephone),
	  0,
	  (GInstanceInitFunc) rubrica_telephone_init
	};

      tel_num_type = g_type_register_static (G_TYPE_OBJECT, "RubricaTelephone",
					     &tel_num_info, 0);
    }
  
  return tel_num_type;
}


static void
rubrica_telephone_class_init(RubricaTelephoneClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_telephone_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_telephone_finalize;
}


static void
rubrica_telephone_init(RubricaTelephone* self)
{
  self->number = NULL;
  self->type   = NULL;

  self->private = g_new(RubricaTelephonePrivate, 1);
  self->private->dispose_has_run = FALSE;
}


static void 
rubrica_telephone_dispose (RubricaTelephone* self)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));
  
#ifdef DEBUG
  g_print("\n\t\trubrica_telephone_dispose");
#endif

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_telephone_finalize (RubricaTelephone* self)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

#ifdef DEBUG
  g_print("\n\t\trubrica_telephone_finalize");
#endif

  rubrica_utils_free_string(self->number);
  rubrica_utils_free_string(self->type);

  g_free(self->private);
  self->private = NULL;
}


RubricaTelephone*
rubrica_telephone_new(void)
{
  RubricaTelephone* self;

  self = g_object_new(rubrica_telephone_get_type(), NULL);

  return self;
}


void
rubrica_telephone_free(RubricaTelephone* self)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

#ifdef DEBUG
  g_print("\n\t\t**** Free Telephone");
#endif

  g_object_unref(self);   
}



void   
rubrica_telephone_set (RubricaTelephone *self, gchar* number, gchar* type)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

  rubrica_telephone_set_number(self, number);
  rubrica_telephone_set_type(self, type);   
}

void   
rubrica_telephone_set_number (RubricaTelephone *self, gchar* number)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

  rubrica_utils_free_string(self->number);

  self->number = g_strdup(number);
}

void   
rubrica_telephone_set_type (RubricaTelephone *self, gchar* type)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

  rubrica_utils_free_string(self->type);  

  self->type = g_strdup(type);
}

void   
rubrica_telephone_get (RubricaTelephone *self, gchar** number, gchar** type)
{
  g_return_if_fail(IS_RUBRICA_TELEPHONE(self));

  if (**number)
    *number = g_strdup(rubrica_telephone_get_number(self));
  if (**type)
    *type = g_strdup(rubrica_telephone_get_tel_type(self));
}


gchar* 
rubrica_telephone_get_number (RubricaTelephone *self)
{
  g_return_val_if_fail(IS_RUBRICA_TELEPHONE(self), NULL);

  return self->number;
}

gchar* 
rubrica_telephone_get_tel_type (RubricaTelephone *self)
{
  g_return_val_if_fail(IS_RUBRICA_TELEPHONE(self), NULL);

  return self->type;
}



RubricaTelephone* 
rubrica_telephone_copy (RubricaTelephone *self)
{
  RubricaTelephone* new;

  g_return_val_if_fail(IS_RUBRICA_TELEPHONE(self), NULL);

  new = rubrica_telephone_new();

  rubrica_telephone_set(new, 
			rubrica_telephone_get_number(self),
			rubrica_telephone_get_tel_type(self));

  return new;
}
