/*
*  RAL -- Rubrica Addressbook Library
*  file: work.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <glib.h>
#include <glib-object.h>

#include "work.h"
#include "utils.h"


struct _RubricaWorkPrivate {
  gboolean dispose_has_run;
};



static void rubrica_work_class_init (RubricaWorkClass* klass);
static void rubrica_work_init       (RubricaWork* obj);

static void rubrica_work_dispose    (RubricaWork* obj);
static void rubrica_work_finalize   (RubricaWork* obj);


GType
rubrica_work_get_type()
{
  static GType work_type = 0;
  
  if (!work_type)
    {
      static const GTypeInfo work_info =
	{
	  sizeof(RubricaWorkClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_work_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaWork),
	  0,
	  (GInstanceInitFunc) rubrica_work_init
	};

      work_type = g_type_register_static (G_TYPE_OBJECT, "RubricaWork",
					  &work_info, 0);
    }
  
  return work_type;
}


static void
rubrica_work_class_init(RubricaWorkClass* klass)
{
  GObjectClass *class;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) rubrica_work_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_work_finalize;
}


static void
rubrica_work_init(RubricaWork* self)
{
  self->assigment    = NULL;
  self->org          = NULL;
  self->dep          = NULL;
  self->subdep       = NULL;
  self->manager      = NULL;
  self->mphone       = NULL;
  self->collaborator = NULL;
  self->cphone       = NULL;

  self->private = g_new(RubricaWorkPrivate, 1);
  self->private->dispose_has_run = FALSE;
}



static void 
rubrica_work_dispose (RubricaWork* self)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));
  
#ifdef DEBUG
  g_print("\n\trubrica_work_dispose");
#endif

  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_work_finalize (RubricaWork* self)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->assigment);
  rubrica_utils_free_string(self->org);
  rubrica_utils_free_string(self->dep);
  rubrica_utils_free_string(self->subdep);
  rubrica_utils_free_string(self->manager);
  rubrica_utils_free_string(self->mphone);
  rubrica_utils_free_string(self->collaborator);
  rubrica_utils_free_string(self->cphone);

  g_free(self->private);
  self->private = NULL;
}


RubricaWork*
rubrica_work_new(void)
{
  RubricaWork* self;

  self = g_object_new(rubrica_work_get_type(), NULL);

  return self;
}


void
rubrica_work_free(RubricaWork* self)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  g_object_unref(self);   
}



void 
rubrica_work_set (RubricaWork* self, gchar* assigment, gchar* org, gchar* dep,
		  gchar* subdep, gchar* manager, gchar* mtelephone, 
		  gchar* collaborator, gchar* ctelephone)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_work_set_assigment(self, assigment);
  rubrica_work_set_organization(self, org);
  rubrica_work_set_department(self, dep);
  rubrica_work_set_subdepartment(self, subdep);
  rubrica_work_set_manager(self, manager);
  rubrica_work_set_mtelephone(self, mtelephone);  
  rubrica_work_set_collaborator(self, collaborator);
  rubrica_work_set_ctelephone(self, ctelephone);
}


void 
rubrica_work_set_assigment (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->assigment);

  self->assigment = g_strdup(str);
}


void 
rubrica_work_set_organization (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->org);

  self->org = g_strdup(str);
}


void 
rubrica_work_set_department (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->dep);

  self->dep = g_strdup(str);
}


void 
rubrica_work_set_subdepartment (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->subdep);

  self->subdep = g_strdup(str);
}


void 
rubrica_work_set_manager (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->manager);
  self->manager = g_strdup(str);
}


void 
rubrica_work_set_mtelephone (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->mphone);
  self->mphone = g_strdup(str);
}

void 
rubrica_work_set_collaborator (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->collaborator);
  self->collaborator = g_strdup(str);
}


void 
rubrica_work_set_ctelephone (RubricaWork* self, gchar* str)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));

  rubrica_utils_free_string(self->cphone);
  self->cphone = g_strdup(str);
}


void   
rubrica_work_get (RubricaWork* self,
		  gchar** assigment, gchar** org, 
		  gchar** dep, gchar** subdep, 
		  gchar** manager, gchar** mtel,
		  gchar** collaborator, gchar** ctel)
{
  g_return_if_fail(IS_RUBRICA_WORK(self));
  
  if (assigment)
    *assigment = rubrica_work_get_assigment(self);
  if (org)
    *org = rubrica_work_get_organization(self);
  if (dep)
    *dep = rubrica_work_get_department(self);
  if (subdep)
    *subdep = rubrica_work_get_subdepartment(self);
  if (manager)
    *manager = rubrica_work_get_manager(self);
  if (mtel)
    *mtel = rubrica_work_get_mtelephone(self);
  if (collaborator)
    *collaborator = rubrica_work_get_collaborator(self);
  if (ctel)
    *ctel = rubrica_work_get_ctelephone(self);
}


gchar* 
rubrica_work_get_assigment (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->assigment;
}


gchar* 
rubrica_work_get_organization (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->org;
}


gchar* 
rubrica_work_get_department (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->dep;
}


gchar* 
rubrica_work_get_subdepartment (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->subdep;
}


gchar* 
rubrica_work_get_manager (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);
  
  return self->manager;
}


gchar* 
rubrica_work_get_mtelephone (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->mphone;
}

gchar* 
rubrica_work_get_collaborator (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);
  
  return self->collaborator;
}


gchar* 
rubrica_work_get_ctelephone (RubricaWork* self)
{
  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  return self->cphone;
}


RubricaWork* 
rubrica_work_copy (RubricaWork* self)
{
  RubricaWork* new;

  g_return_val_if_fail(IS_RUBRICA_WORK(self), NULL);

  new = rubrica_work_new();

  rubrica_work_set(new, 
		   rubrica_work_get_assigment(self),
		   rubrica_work_get_organization(self),
		   rubrica_work_get_department(self),
		   rubrica_work_get_subdepartment(self),
		   rubrica_work_get_manager(self),
		   rubrica_work_get_mtelephone(self),
		   rubrica_work_get_collaborator(self),
		   rubrica_work_get_ctelephone(self));

  return new;
}

